"""
Lpstat - command ``lpstat``
===========================

Parsers contains in this module are:

LpstatPrinters - command ``/usr/bin/lpstat -p``

LpstatProtocol - command ``/usr/bin/lpstat -v``

LpstatQueuedJobs - command ``/usr/bin/lpstat -o``
"""
from insights.core import CommandParser, Parser
from insights.core.exceptions import SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs

# Printer states
PRINTER_STATUS_IDLE = 'IDLE'
PRINTER_STATUS_PROCESSING = 'PROCESSING'
PRINTER_STATUS_DISABLED = 'DISABLED'
PRINTER_STATUS_UNKNOWN = 'UNKNOWN'

START_LINE_MARKER = 'printer '


@parser(Specs.lpstat_p)
class LpstatPrinters(CommandParser):
    """
    Class to parse ``lpstat -p`` command output. Parses the output of ``lpstat -p``, to get locally configured printers.
    Current available printer states are:

    * IDLE (``PRINTER_STATUS_IDLE``)
    * PROCESSING (``PRINTER_STATUS_PROCESSING``) -- printing
    * DISABLED (``PRINTER_STATUS_DISABLED``)
    * UNKNOWN (``PRINTER_STATUS_UNKNOWN``)

    Sample output of the command:

        printer idle_printer is idle.  enabled since Fri 20 Jan 2017 09:55:50 PM CET
        printer disabled_printer disabled since Wed 15 Feb 2017 12:01:11 PM EST -
        reason unknown

    Raises:
        ValueError: Raised if any error occurs parsing the content.

    Examples:
    >>> type(lpstat_printers)
    <class 'insights.parsers.lpstat.LpstatPrinters'>
    >>> len(lpstat_printers.printers)
    3
    >>> lpstat_printers.printer_names_by_status('DISABLED')
    ['disabled_printer']
    """

    def __init__(self, *args, **kwargs):
        self.printers = []
        """dict: Dictionary of locally configured printers, with keys 'name' and 'status'"""
        super(LpstatPrinters, self).__init__(*args, **kwargs)

    def parse_content(self, content):
        marker_len = len(START_LINE_MARKER)
        for line in content:
            if line.startswith(START_LINE_MARKER):
                printer_line = line[marker_len:]
                # cut printer name until next space character
                printer_name = printer_line[:printer_line.index(' ')]
                state_line_starts = marker_len + len(printer_name) + 1  # 1 is space
                state_line = line[state_line_starts:]

                printer = {
                    'name': printer_name,
                    'status': self._parse_status(state_line)
                }
                self.printers.append(printer)

    def _parse_status(self, state_line):
        if 'is idle' in state_line:
            return PRINTER_STATUS_IDLE
        elif 'printing' in state_line:
            return PRINTER_STATUS_PROCESSING
        elif 'disabled' in state_line:
            return PRINTER_STATUS_DISABLED

        return PRINTER_STATUS_UNKNOWN

    def printer_names_by_status(self, status):
        """Gives names of configured printers for a given status

        Arguments:
            status (string)
        """
        names = [prntr['name'] for prntr in self.printers if prntr['status'] == status]
        return names


@parser(Specs.lpstat_protocol_printers)
class LpstatProtocol(CommandParser, dict):
    """
    Class to parse ``lpstat -v`` command output.

    Sample output of the command::

        device for test_printer1: ipp
        device for test_printer2: ipp
        device for savtermhpc: implicitclass:savtermhpc
        device for A1: marshaA1:/tmp/A1

    Examples:
        >>> type(lpstat_protocol)
        <class 'insights.parsers.lpstat.LpstatProtocol'>
        >>> lpstat_protocol['test_printer1']
        'ipp'
        >>> lpstat_protocol['savtermhpc']
        'implicitclass'
    """
    def parse_content(self, content):
        if not content:
            raise SkipComponent("No Valid Output")
        data = {}
        for line in content:
            if line.startswith("device for "):
                line_split = line.split(":")
                protocol = line_split[1].strip()
                printer = line_split[0].split()[-1].strip()
                data[printer] = protocol
        if not data:
            raise SkipComponent("No Valid Output")
        self.update(data)


@parser(Specs.lpstat_queued_jobs_count)
class LpstatQueuedJobs(Parser):
    """
    Class to parse lpstat_queued_jobs_count datasource result to get the count of queued jobs.

    Currently, this parser only provide one attribute count which returns the count of queued
    jobs but no detailed jobs are provided.

    Sample output of the ``/usr/bin/lpstat -o`` command::

        Cups-PDF-1802           root          265443328   Tue 05 Sep 2023 02:21:19 PM CST
        Cups-PDF-1803           root          265443328   Tue 05 Sep 2023 02:21:21 PM CST
        Cups-PDF-1804           root          265443328   Tue 05 Sep 2023 02:21:22 PM CST

    Examples:
        >>> type(lpstat_queued_jobs)
        <class 'insights.parsers.lpstat.LpstatQueuedJobs'>
        >>> lpstat_queued_jobs.count
        3
    """
    def parse_content(self, content):
        if not content:
            raise SkipComponent("Empty result")
        self.count = int(content[0])



from insights.contrib import ipaddress
from insights.parsers import ip
from insights.tests import context_wrap
from insights.util import keys_in


IP_ADDR_TEST = """
Message truncated
Message truncated
Message truncated
1: lo: <LOOPBACK,UP,LOWER_UP> mtu 16436 qdisc noqueue state UNKNOWN
    link/loopback 00:00:00:00:00:00 brd 00:00:00:00:00:00
    inet 127.0.0.1/8 scope host lo
    inet6 ::1/128 scope host
    valid_lft forever preferred_lft forever
2: eth7: <NO-CARRIER,BROADCAST,MULTICAST,SLAVE,UP> mtu 1500 qdisc mq master bond1 state DOWN qlen 1000
    link/ether 00:11:3f:e2:f5:9f brd ff:ff:ff:ff:ff:ff link-netnsid 1
3: tunl0: <NOARP> mtu 1480 qdisc noop state DOWN
    link/ipip 0.0.0.0 brd 0.0.0.0
4: tunl1: <NOARP> mtu 1480 qdisc noop state DOWN
    link/[65534]
    inet 172.30.0.1 peer 172.30.0.2/32 scope global tun0
5: bond1.57@bond1: <BROADCAST,MULTICAST,MASTER,UP,LOWER_UP> mtu 1500 qdisc noqueue state UP
    link/ether 00:11:3f:e2:f5:9e brd ff:ff:ff:ff:ff:ff
    inet 10.192.4.171/27 brd 10.192.4.191 scope global bond1.57
    inet6 fe80::211:3fff:fee2:f59e/64 scope link
    valid_lft forever preferred_lft forever
    inet6 2001::211:3fff:fee2:f59e/64 scope global mngtmpaddr dynamic
    valid_lft 2592000sec preferred_lft 6480000sec
6: ip.tun2: <POINTOPOINT,NOARP,UP,LOWER_UP> mtu 80 qdisc noqueue state UNKNOWN
    link/ipip 10.192.4.203 peer 10.188.61.108
    inet 192.168.112.5 peer 192.168.122.6/32 scope global ip.tun2
""".strip()


def test_ip_addr():
    d = ip.IpAddr(context_wrap(IP_ADDR_TEST))

    assert len(d) == 6
    assert keys_in(["lo", "eth7", "tunl0", "tunl1", "bond1.57", "ip.tun2"], d)

    assert sorted(d.active) == sorted(['lo', 'eth7', 'bond1.57', 'ip.tun2'])

    lo = d["lo"]
    assert len(lo) == 2
    assert lo["mac"] == "00:00:00:00:00:00"
    assert lo["flags"] == ["LOOPBACK", "UP", "LOWER_UP"]
    assert lo["type"] == "loopback"
    assert lo["mac"] == "00:00:00:00:00:00"
    assert lo["mtu"] == 16436
    assert lo.addrs(4) == ["127.0.0.1"]
    assert lo.addrs(6) == ["::1"]

    eth7 = d["eth7"]
    assert len(eth7) == 0
    assert eth7["mac"] == "00:11:3f:e2:f5:9f"
    assert eth7["flags"] == ["NO-CARRIER", "BROADCAST", "MULTICAST", "SLAVE", "UP"]
    assert eth7["master"] == "bond1"
    assert eth7["qlen"] == 1000
    assert eth7["mtu"] == 1500
    assert len(eth7["addr"]) == 0

    tunl0 = d["tunl0"]
    assert tunl0["type"] == "ipip"

    tunl1 = d["tunl1"]
    assert tunl1["type"] == "[65534]"
    assert tunl1.addrs() == ["172.30.0.2"]  # P2P links use the remote IP

    bond1Addr = d["bond1.57"]
    assert len(bond1Addr) == 3
    assert bond1Addr.addrs(4) == ["10.192.4.171"]
    assert bond1Addr.addrs(6) == ["fe80::211:3fff:fee2:f59e", "2001::211:3fff:fee2:f59e"]

    tun2 = d["ip.tun2"]
    assert tun2["mtu"] == 80
    assert tun2["type"] == "ipip"
    assert tun2["peer"] == "10.188.61.108"
    assert tun2["peer_ip"] == "10.192.4.203"
    assert tun2.addrs(4) == ["192.168.122.6"]

    # Tests of __cmp__
    assert eth7 != tunl0
    assert tunl0 != tunl1
    assert tunl1 == d['tunl1']

    # Tests of __iter__ and __getitem__
    for iface in d:
        assert iface is not None


IP_S_LINK = """
1: enp0s3: <BROADCAST,MULTICAST,UP,LOWER_UP> mtu 1500 qdisc pfifo_fast state UP mode DEFAULT qlen 1000
    link/ether 08:00:27:4a:c5:ef brd ff:ff:ff:ff:ff:ff
    RX: bytes  packets  errors  dropped overrun mcast
    1113685    2244     0       0       0       0
    TX: bytes  packets  errors  dropped carrier collsns
    550754     1407     0       0       0       0
2: lo: <LOOPBACK,UP,LOWER_UP> mtu 65536 qdisc noqueue state UNKNOWN mode DEFAULT
    link/loopback 00:00:00:00:00:00 brd 00:00:00:00:00:00
    RX: bytes  packets  errors  dropped overrun mcast
    884        98       0       0       0       0
    TX: bytes  packets  errors  dropped carrier collsns
    884        10       0       0       0       0
3: enp0s8: <BROADCAST,MULTICAST,UP,LOWER_UP> mtu 1500 qdisc pfifo_fast state UP mode DEFAULT qlen 1000
    link/ether 08:00:27:db:86:9e brd ff:ff:ff:ff:ff:ff
    RX: bytes  packets  errors  dropped overrun mcast
    0          1        0       0       0       0
    TX: bytes  packets  errors  dropped carrier collsns
    0          4        0       0       0       0
4: enp0s9: <BROADCAST,UP,MULTICAST> mtu 1500 qdisc pfifo_fast state UP mode DEFAULT qlen 1000
    link/ether 08:00:27:a6:bd:65 brd ff:ff:ff:ff:ff:ff
    RX: bytes  packets  errors  dropped overrun mcast
    0          8        0       0       0       0
    TX: bytes  packets  errors  dropped carrier collsns
    0          12        0       0       0       0
5: tm0: <BROADCAST,MULTICAST,UP,LOWER_UP> mtu 1500 qdisc noqueue master br0 state UP mode DEFAULT group default qlen 1000
    link/ether d4:f5:ef:01:1a:3c brd ff:ff:ff:ff:ff:ff promiscuity 1
    team
    bridge_slave state forwarding priority 32 cost 100 hairpin off guard off root_block off fastleave off learning on flood on port_id 0x8001 port_no 0x1 designated_port 32769 designated_cost 0 designated_bridge 8000.d4:f5:ef:1:1a:3c designated_root 8000.d4:f5:ef:1:1a:3c hold_timer    0.00 message_age_timer    0.00 forward_delay_timer    0.00 topology_change_ack 0 config_pending 0 proxy_arp off proxy_arp_wifi off mcast_router 1 mcast_fast_leave off mcast_flood on neigh_suppress off group_fwd_mask 0x0 group_fwd_mask_str 0x0 vlan_tunnel off addrgenmode none numtxqueues 16 numrxqueues 16 gso_max_size 65536 gso_max_segs 65535
    RX: bytes  packets  errors  dropped overrun mcast
    959425     11784    0       0       0       1
    TX: bytes  packets  errors  dropped carrier collsns
    2980335    14289    0       0       0       0
""".strip()

IP_S_LINK_ALL = """
1: lo: <LOOPBACK,UP,LOWER_UP> mtu 65536 qdisc noqueue state UNKNOWN mode DEFAULT group default qlen 1000
    link/loopback 00:00:00:00:00:00 brd 00:00:00:00:00:00
    RX: bytes  packets  errors  dropped overrun mcast
    100608     1208     0       0       0       0
    TX: bytes  packets  errors  dropped carrier collsns
    100608     1208     0       0       0       0
2: enp0s25: <BROADCAST,MULTICAST,UP,LOWER_UP> mtu 1500 qdisc fq_codel state UP mode DEFAULT group default qlen 1000
    link/ether 1c:75:08:a5:7e:25 brd ff:ff:ff:ff:ff:ff
    RX: bytes  packets  errors  dropped overrun mcast
    1112747593 1492476  0       72813   0       7
    TX: bytes  packets  errors  dropped carrier collsns
    216483610  969514   0       0       0       0
3: wlp3s0: <BROADCAST,MULTICAST> mtu 1500 qdisc noop state DOWN mode DEFAULT group default qlen 1000
    link/ether 86:62:bd:26:12:63 brd ff:ff:ff:ff:ff:ff
    RX: bytes  packets  errors  dropped overrun mcast
    0          0        0       0       0       0
    TX: bytes  packets  errors  dropped carrier collsns
    0          0        0       0       0       0
4: virbr0: <BROADCAST,MULTICAST,UP,LOWER_UP> mtu 1500 qdisc noqueue state UP mode DEFAULT group default qlen 1000
    link/ether 52:54:00:17:84:7d brd ff:ff:ff:ff:ff:ff
    RX: bytes  packets  errors  dropped overrun mcast
    48067      769      0       4       0       0
    TX: bytes  packets  errors  dropped carrier collsns
    120681     1022     0       0       0       0
5: virbr0-nic: <BROADCAST,MULTICAST> mtu 1500 qdisc fq_codel master virbr0 state DOWN mode DEFAULT group default qlen 1000
    link/ether 52:54:00:17:84:7d brd ff:ff:ff:ff:ff:ff
    RX: bytes  packets  errors  dropped overrun mcast
    0          0        0       0       0       0
    TX: bytes  packets  errors  dropped carrier collsns
    0          0        0       0       0       0
6: vnet0: <BROADCAST,MULTICAST,UP,LOWER_UP> mtu 1500 qdisc fq_codel master virbr0 state UNKNOWN mode DEFAULT group default qlen 1000
    link/ether fe:54:00:43:7e:91 brd ff:ff:ff:ff:ff:ff
    RX: bytes  packets  errors  dropped overrun mcast
    58833      769      0       0       0       0
    TX: bytes  packets  errors  dropped carrier collsns
    605233     10237    0       0       0       0
7: ppp0: <POINTOPOINT,MULTICAST,NOARP,UP,LOWER_UP> mtu 1492 qdisc fq_codel state UNKNOWN mode DEFAULT group default qlen 3
    link/ppp
    RX: bytes  packets  errors  dropped overrun mcast
    1070945361 1417562  0       0       0       0
    TX: bytes  packets  errors  dropped carrier collsns
    191198320  967571   0       0       0       0
8: tun0: <POINTOPOINT,MULTICAST,NOARP,UP,LOWER_UP> mtu 1360 qdisc fq_codel state UNKNOWN mode DEFAULT group default qlen 100
    link/none
    RX: bytes  packets  errors  dropped overrun mcast
    128212905  157277   0       0       0       0
    TX: bytes  packets  errors  dropped carrier collsns
    12341814    136884  0       0       0       0
""".strip()

IP_S_LINK_ALL_2 = """
1: lo: <LOOPBACK,UP,LOWER_UP> mtu 65536 qdisc noqueue state UNKNOWN mode DEFAULT group default qlen 1000
    link/loopback 00:00:00:00:00:00 brd 00:00:00:00:00:00 promiscuity 0 addrgenmode eui64 numtxqueues 1 numrxqueues 1 gso_max_size 65536 gso_max_segs 65535
    RX: bytes  packets  errors  dropped overrun mcast
    38374458039 291293530 0       0       0       0
    TX: bytes  packets  errors  dropped carrier collsns
    38374458039 291293530 0       0       0       0
2: eno1: <BROADCAST,MULTICAST,UP,LOWER_UP> mtu 9000 qdisc mq master ovs-system state UP mode DEFAULT group default qlen 1000
    link/ether 38:90:a5:80:43:c9 brd ff:ff:ff:ff:ff:ff promiscuity 1
    openvswitch_slave addrgenmode eui64 numtxqueues 8 numrxqueues 8 gso_max_size 65536 gso_max_segs 65535
    RX: bytes  packets  errors  dropped overrun mcast
    108341661705196 87334303936 0       5632499 0       7110182
    TX: bytes  packets  errors  dropped carrier collsns
    297580787699558 95024371467 0       0       0       0
3: eno2: <BROADCAST,MULTICAST,UP,LOWER_UP> mtu 9000 qdisc mq master ovs-system state UP mode DEFAULT group default qlen 1000
    link/ether 38:90:a5:80:43:ca brd ff:ff:ff:ff:ff:ff promiscuity 1
    openvswitch_slave addrgenmode eui64 numtxqueues 8 numrxqueues 8 gso_max_size 65536 gso_max_segs 65535
    RX: bytes  packets  errors  dropped overrun mcast
    169323904323594 116776818590 0       38367215 0       7111574
    TX: bytes  packets  errors  dropped carrier collsns
    257318435088822 64965403562 0       0       0       0
4: ovs-system: <BROADCAST,MULTICAST,UP,LOWER_UP> mtu 1500 qdisc noop state DOWN mode DEFAULT group default qlen 1000
    link/ether d6:fe:ce:4d:91:c1 brd ff:ff:ff:ff:ff:ff promiscuity 1
    openvswitch addrgenmode eui64 numtxqueues 1 numrxqueues 1 gso_max_size 65536 gso_max_segs 65535
    RX: bytes  packets  errors  dropped overrun mcast
    0          0        0       0       0       0
    TX: bytes  packets  errors  dropped carrier collsns
    0          0        0       0       0       0
5: br-ex: <BROADCAST,MULTICAST,UP,LOWER_UP> mtu 1500 qdisc noop state DOWN mode DEFAULT group default qlen 1000
    link/ether 32:0f:11:3e:4f:49 brd ff:ff:ff:ff:ff:ff promiscuity 1
    openvswitch addrgenmode eui64 numtxqueues 1 numrxqueues 1 gso_max_size 65536 gso_max_segs 65535
    RX: bytes  packets  errors  dropped overrun mcast
    0          0        0       0       0       0
    TX: bytes  packets  errors  dropped carrier collsns
    0          0        0       0       0       0
6: vxlan_sys_4789: <BROADCAST,MULTICAST,UP,LOWER_UP> mtu 65000 qdisc noqueue master ovs-system state UNKNOWN mode DEFAULT group default qlen 1000
    link/ether aa:be:45:24:75:f8 brd ff:ff:ff:ff:ff:ff promiscuity 1
    vxlan id 0 srcport 0 0 dstport 4789 nolearning ageing 300 noudpcsum noudp6zerocsumtx udp6zerocsumrx external
    openvswitch_slave addrgenmode eui64 numtxqueues 1 numrxqueues 1 gso_max_size 65536 gso_max_segs 65535
    RX: bytes  packets  errors  dropped overrun mcast
    209941850791128 169319683635 0       0       0       0
    TX: bytes  packets  errors  dropped carrier collsns
    89501939148105 38904599752 0       87515   0       0
""".strip()

IP_S_LINK_ALL_3 = """
1: lo: <LOOPBACK,UP,LOWER_UP> mtu 65536 qdisc noqueue state UNKNOWN mode DEFAULT group default qlen 1000
    link/loopback 00:00:00:00:00:00 brd 00:00:00:00:00:00 promiscuity 0 addrgenmode eui64 numtxqueues 1 numrxqueues 1 gso_max_size 65536 gso_max_segs 65535
    RX: bytes  packets  errors  dropped overrun mcast
    18951865334 117657102 0       0       0       0
    TX: bytes  packets  errors  dropped carrier collsns
    18951865334 117657102 0       0       0       0
2: eth0: <BROADCAST,MULTICAST> mtu 1500 qdisc noop state DOWN mode DEFAULT group default qlen 1000
    link/ether 0c:c4:7a:4d:84:b4 brd ff:ff:ff:ff:ff:ff promiscuity 0 addrgenmode eui64 numtxqueues 8 numrxqueues 8 gso_max_size 65536 gso_max_segs 65535
    RX: bytes  packets  errors  dropped overrun mcast
    0          0        0       0       0       0
    TX: bytes  packets  errors  dropped carrier collsns
    0          0        0       0       0       0
3: eth1: <BROADCAST,MULTICAST> mtu 1500 qdisc noop state DOWN mode DEFAULT group default qlen 1000
    link/ether 0c:c4:7a:4d:84:b5 brd ff:ff:ff:ff:ff:ff promiscuity 0 addrgenmode eui64 numtxqueues 8 numrxqueues 8 gso_max_size 65536 gso_max_segs 65535
    RX: bytes  packets  errors  dropped overrun mcast
    0          0        0       0       0       0
    TX: bytes  packets  errors  dropped carrier collsns
    0          0        0       0       0       0
4: eth0_1: <BROADCAST,MULTICAST,SLAVE,UP,LOWER_UP> mtu 9000 qdisc mq master bond0 state UP mode DEFAULT group default qlen 1000
    link/ether 00:90:fa:8d:36:1e brd ff:ff:ff:ff:ff:ff promiscuity 1
    bond_slave state ACTIVE mii_status UP link_failure_count 0 perm_hwaddr 00:90:fa:8d:36:1e queue_id 0 ad_aggregator_id 1 ad_actor_oper_port_state 63 ad_partner_oper_port_state 61 addrgenmode eui64 numtxqueues 32 numrxqueues 32 gso_max_size 65513 gso_max_segs 65535 portid 010000000041303830343437334650434e
    RX: bytes  packets  errors  dropped overrun mcast
    16809815066893 15139642734 0       117902  0       863246747
    TX: bytes  packets  errors  dropped carrier collsns
    35867456371871 19520904636 0       0       0       0
5: eth0_2: <BROADCAST,MULTICAST,SLAVE,UP,LOWER_UP> mtu 9000 qdisc mq master bond0 state UP mode DEFAULT group default qlen 1000
    link/ether 00:90:fa:8d:36:1e brd ff:ff:ff:ff:ff:ff promiscuity 1
    bond_slave state ACTIVE mii_status UP link_failure_count 0 perm_hwaddr 00:90:fa:8d:36:22 queue_id 0 ad_aggregator_id 1 ad_actor_oper_port_state 63 ad_partner_oper_port_state 61 addrgenmode eui64 numtxqueues 32 numrxqueues 32 gso_max_size 65513 gso_max_segs 65535 portid 020000000041303830343437334650434e
    RX: bytes  packets  errors  dropped overrun mcast
    15566486160064 16375611024 0       117902  0       509010362
    TX: bytes  packets  errors  dropped carrier collsns
    41939419563431 12260064477 0       0       0       0
6: ovs-system: <BROADCAST,MULTICAST> mtu 1500 qdisc noop state DOWN mode DEFAULT group default qlen 1000
    link/ether 0e:2c:25:b4:71:7f brd ff:ff:ff:ff:ff:ff promiscuity 1
    openvswitch addrgenmode eui64 numtxqueues 1 numrxqueues 1 gso_max_size 65536 gso_max_segs 65535
    RX: bytes  packets  errors  dropped overrun mcast
    0          0        0       0       0       0
    TX: bytes  packets  errors  dropped carrier collsns
    0          0        0       0       0       0
7: vxlan_sys_4789: <BROADCAST,MULTICAST,UP,LOWER_UP> mtu 65520 qdisc noqueue master ovs-system state UNKNOWN mode DEFAULT group default qlen 1000
    link/ether ba:02:63:fe:e4:de brd ff:ff:ff:ff:ff:ff promiscuity 1
    vxlan id 0 srcport 0 0 dstport 4789 nolearning ageing 300 udpcsum noudp6zerocsumtx udp6zerocsumrx external
    openvswitch_slave addrgenmode eui64 numtxqueues 1 numrxqueues 1 gso_max_size 65536 gso_max_segs 65535
    RX: bytes  packets  errors  dropped overrun mcast
    18615536781687 18438893425 0       0       0       0
    TX: bytes  packets  errors  dropped carrier collsns
    14770242373297 14414656441 0       0       0       0
8: br-tun: <BROADCAST,MULTICAST> mtu 1500 qdisc noop state DOWN mode DEFAULT group default qlen 1000
    link/ether 42:31:7a:81:8e:4c brd ff:ff:ff:ff:ff:ff promiscuity 1
    openvswitch addrgenmode eui64 numtxqueues 1 numrxqueues 1 gso_max_size 65536 gso_max_segs 65535
    RX: bytes  packets  errors  dropped overrun mcast
    0          0        0       0       0       0
    TX: bytes  packets  errors  dropped carrier collsns
    0          0        0       0       0       0
9: br-int: <BROADCAST,MULTICAST> mtu 8950 qdisc noop state DOWN mode DEFAULT group default qlen 1000
    link/ether fa:cf:5f:aa:34:44 brd ff:ff:ff:ff:ff:ff promiscuity 1
    openvswitch addrgenmode eui64 numtxqueues 1 numrxqueues 1 gso_max_size 65536 gso_max_segs 65535
    RX: bytes  packets  errors  dropped overrun mcast
    0          0        0       101259804 0     0
    TX: bytes  packets  errors  dropped carrier collsns
    0          0        0       0       0       0
10: gre0@NONE: <NOARP> mtu 1476 qdisc noqueue state DOWN mode DEFAULT qlen 1
    link/gre 0.0.0.0 brd 0.0.0.0 promiscuity 0
    gre remote any local any ttl inherit nopmtudisc addrgenmode eui64
    RX: bytes  packets  errors  dropped overrun mcast
    0          0        0       0       0       0
    TX: bytes  packets  errors  dropped carrier collsns
    0          0        0       0       0       0
11: gretap0@NONE: <BROADCAST,MULTICAST> mtu 1462 qdisc noop state DOWN mode DEFAULT qlen 1000
    link/ether 00:00:00:00:00:00 brd ff:ff:ff:ff:ff:ff promiscuity 0
    gretap remote any local any ttl inherit nopmtudisc addrgenmode eui64
    RX: bytes  packets  errors  dropped overrun mcast
    0          0        0       0       0       0
    TX: bytes  packets  errors  dropped carrier collsns
    0          0        0       0       0       0
12: gre1@NONE: <POINTOPOINT,NOARP,UP,LOWER_UP> mtu 1476 qdisc noqueue state UNKNOWN mode DEFAULT qlen 1
    link/gre 10.211.47.217 peer 172.168.10.25 promiscuity 0
    gre remote 172.168.10.25 local 10.211.47.217 ttl 255 addrgenmode eui64
    RX: bytes  packets  errors  dropped overrun mcast
    0          0        0       0       0       0
    TX: bytes  packets  errors  dropped carrier collsns
    168        3        0       0       0       0
13: geneve0: <BROADCAST,MULTICAST,UP,LOWER_UP> mtu 1500 qdisc noqueue state UNKNOWN mode DEFAULT qlen 1000
    link/ether 8e:a3:1d:43:d6:d9 brd ff:ff:ff:ff:ff:ff promiscuity 0
    geneve id 10 remote 192.168.43.254 dstport 6081 noudpcsum udp6zerocsumrx addrgenmode eui64
    RX: bytes  packets  errors  dropped overrun mcast
    0          0        0       0       0       0
    TX: bytes  packets  errors  dropped carrier collsns
    536        8        0       0       0       0
14: geneve1: <BROADCAST,MULTICAST,UP,LOWER_UP> mtu 1500 qdisc noqueue state UNKNOWN mode DEFAULT qlen 1000
    link/ether 22:14:52:d4:a2:13 brd ff:ff:ff:ff:ff:ff promiscuity 0
    geneve id 11 remote 192.168.43.173 dstport 6081 noudpcsum udp6zerocsumrx addrgenmode eui64
    RX: bytes  packets  errors  dropped overrun mcast
    0          0        8       0       0       0
    TX: bytes  packets  errors  dropped carrier collsns
    536        8        0       0       0       0
""".strip()

IP_S_LINK_ALL_4 = """
1: lo: <LOOPBACK,UP,LOWER_UP> mtu 65536 qdisc noqueue state UNKNOWN mode DEFAULT group default qlen 1000
    link/loopback 00:00:00:00:00:00 brd 00:00:00:00:00:00 promiscuity 0 minmtu 0 maxmtu 0 addrgenmode eui64 numtxqueues 1 numrxqueues 1 gso_max_size 65536 gso_max_segs 65535
    RX: bytes  packets  errors  dropped missed  mcast
    10409771   7627     0       0       0       0
    TX: bytes  packets  errors  dropped carrier collsns
    10409771   7627     0       0       0       0
3: ens1f0: <BROADCAST,MULTICAST,SLAVE,UP,LOWER_UP> mtu 1500 qdisc mq master bond1 state UP mode DEFAULT group default qlen 1000
    link/ether b4:96:91:e5:c8:a8 brd ff:ff:ff:ff:ff:ff promiscuity 1 minmtu 68 maxmtu 9702
    bond_slave state ACTIVE mii_status UP link_failure_count 0 perm_hwaddr b4:96:91:e5:c8:a8 queue_id 0 addrgenmode none numtxqueues 112 numrxqueues 112 gso_max_size 65536 gso_max_segs 65535
    RX: bytes  packets  errors  dropped missed  mcast
    3486063351 2629875  0       0       0       3064
    TX: bytes  packets  errors  dropped carrier collsns
    259638429  543551   0       0       0       0
7: ens1f1: <BROADCAST,MULTICAST,UP,LOWER_UP> mtu 1500 qdisc mq state UP mode DEFAULT group default qlen 1000
    link/ether b4:96:91:e5:c8:a9 brd ff:ff:ff:ff:ff:ff promiscuity 0 minmtu 68 maxmtu 9702 addrgenmode none numtxqueues 112 n
umrxqueues 112 gso_max_size 65536 gso_max_segs 65535
    RX: bytes  packets  errors  dropped missed  mcast
    17496609   257835   0       67      0       792
    TX: bytes  packets  errors  dropped carrier collsns
    33991      231      0       0       0       0
    vf 0     link/ether 6e:cc:c3:fe:ad:7f brd ff:ff:ff:ff:ff:ff, spoof checking on, link-state auto, trust off
    RX: bytes  packets  mcast   bcast   dropped
    14069717   192865   0       192865  192865
    TX: bytes  packets   dropped
    0          0        0
    vf 1     link/ether c6:5b:b6:3c:db:f1 brd ff:ff:ff:ff:ff:ff, spoof checking on, link-state auto, trust off
    RX: bytes  packets  mcast   bcast   dropped
    14069717   192865   0       192865  192865
    TX: bytes  packets   dropped
    0          0        0
    vf 2     link/ether f6:a4:d0:2d:a5:a1 brd ff:ff:ff:ff:ff:ff, spoof checking on, link-state auto, trust off
    RX: bytes  packets  mcast   bcast   dropped
    14069717   192865   0       192865  192865
    TX: bytes  packets   dropped
    0          0        0
    vf 3     link/ether 92:50:97:c4:9d:f0 brd ff:ff:ff:ff:ff:ff, spoof checking on, link-state auto, trust off
    RX: bytes  packets  mcast   bcast   dropped
    14069717   192865   0       192865  192865
    TX: bytes  packets   dropped
    0          0        0
91: ens1f1v64: <BROADCAST,MULTICAST> mtu 1500 qdisc noop state DOWN mode DEFAULT group default qlen 1000
    link/ether 76:47:d1:15:9f:59 brd ff:ff:ff:ff:ff:ff promiscuity 0 minmtu 68 maxmtu 9702 addrgenmode eui64 numtxqueues 16 numrxqueues 16 gso_max_size 65536 gso_max_segs 65535
    RX: bytes  packets  errors  dropped missed  mcast
    0          0        0       0       0       0
    TX: bytes  packets  errors  dropped carrier collsns
    0          0        0       0       0       0
""".strip()

IP_S_LINK_ALL_5 = """
1: lo: <LOOPBACK,UP,LOWER_UP> mtu 65536 qdisc noqueue state UNKNOWN mode DEFAULT group default qlen 1000
    link/loopback 00:00:00:00:00:00 brd 00:00:00:00:00:00 promiscuity 0 minmtu 0 maxmtu 0 addrgenmode eui64 numtxqueues 1 numrxqueues 1 gso_max_size 65536 gso_max_segs 65535
    RX: bytes  packets  errors  dropped missed  mcast
    10409771   7627     0       0       0       0
    TX: bytes  packets  errors  dropped carrier collsns
    10409771   7627     0       0       0       0
2: ens1: <BROADCAST,MULTICAST,SLAVE,UP,LOWER_UP> mtu 1500 qdisc mq master bond1 state UP mode DEFAULT group default qlen 1000 unpaired_attr
    link/ether b4:96:91:e5:c8:a8 brd ff:ff:ff:ff:ff:ff promiscuity 1 minmtu 68 maxmtu 9702
    bond_slave state ACTIVE mii_status UP link_failure_count 0 perm_hwaddr b4:96:91:e5:c8:a8 queue_id 0 addrgenmode none numtxqueues 112 numrxqueues 112 gso_max_size 65536 gso_max_segs 65535
    RX: bytes  packets  errors  dropped missed  mcast
    3486063351 2629875  0       0       0       3064
    TX: bytes  packets  errors  dropped carrier collsns
    259638429  543551   0       0       0       0
""".strip()


def test_ip_data_Link():
    link_info = ip.IpLinkInfo(context_wrap(IP_S_LINK))
    link_info_all = ip.IpLinkInfo(context_wrap(IP_S_LINK_ALL))
    link_info_all_2 = ip.IpLinkInfo(context_wrap(IP_S_LINK_ALL_2))
    link_info_all_3 = ip.IpLinkInfo(context_wrap(IP_S_LINK_ALL_3))
    link_info_all_4 = ip.IpLinkInfo(context_wrap(IP_S_LINK_ALL_4))
    link_info_all_5 = ip.IpLinkInfo(context_wrap(IP_S_LINK_ALL_5))
    if_list_all_3 = link_info_all_3.active
    assert sorted(if_list_all_3) == sorted(['lo', 'eth0_1', 'eth0_2', 'vxlan_sys_4789', 'gre1', 'geneve0', 'geneve1'])
    eth0_1 = link_info_all_3["eth0_1"]
    assert eth0_1["mac"] == "00:90:fa:8d:36:1e"
    assert eth0_1["rx_packets"] == 15139642734
    assert eth0_1["rx_bytes"] == 16809815066893
    if_list_all_2 = link_info_all_2.active
    if_list_all = link_info_all.active
    if_list = link_info.active
    assert len(if_list) == 5

    assert keys_in(["lo", "enp0s3", "enp0s8", "enp0s9", "tm0"], if_list)
    assert keys_in(['ppp0', 'lo', 'tun0', 'enp0s25', 'vnet0', 'virbr0'], if_list_all)
    assert keys_in(['lo', 'eno1', 'eno2', 'ovs-system', 'br-ex', 'vxlan_sys_4789'], if_list_all_2)

    assert sorted(link_info.active) == sorted(['lo', 'enp0s3', 'enp0s8', 'enp0s9', 'tm0'])

    lo = link_info["lo"]
    assert lo["mac"] == "00:00:00:00:00:00"
    assert lo["flags"] == ["LOOPBACK", "UP", "LOWER_UP"]
    assert lo["type"] == "loopback"
    assert lo["mtu"] == 65536
    assert lo["rx_packets"] == 98
    assert lo["tx_packets"] == 10
    assert lo["index"] == 2

    enp0s3 = link_info["enp0s3"]
    assert enp0s3["mac"] == "08:00:27:4a:c5:ef"
    assert enp0s3["flags"] == ["BROADCAST", "MULTICAST", "UP", "LOWER_UP"]
    assert enp0s3["type"] == "ether"
    assert enp0s3["mtu"] == 1500
    assert enp0s3["rx_packets"] == 2244
    assert enp0s3["tx_packets"] == 1407
    assert enp0s3["index"] == 1

    enp0s25 = link_info_all["enp0s25"]
    assert enp0s25["mac"] == "1c:75:08:a5:7e:25"
    assert enp0s25["flags"] == ["BROADCAST", "MULTICAST", "UP", "LOWER_UP"]
    assert enp0s25["type"] == "ether"
    assert enp0s25["mtu"] == 1500
    assert enp0s25["rx_packets"] == 1492476
    assert enp0s25["tx_packets"] == 969514
    assert enp0s25["index"] == 2

    vxlan_sys_4789 = link_info_all_2["vxlan_sys_4789"]
    assert sorted(vxlan_sys_4789["vxlan"]) == sorted(['vxlan', 'id', '0', 'srcport', '0', '0', 'dstport', '4789', 'nolearning', 'ageing', '300', 'noudpcsum', 'noudp6zerocsumtx', 'udp6zerocsumrx', 'external'])
    ovs = link_info_all_2["ovs-system"]
    assert sorted(ovs["openvswitch"]) == sorted(['openvswitch', 'addrgenmode', 'eui64', 'numtxqueues', '1', 'numrxqueues', '1', 'gso_max_size', '65536', 'gso_max_segs', '65535'])
    geneve_obj = link_info_all_3['geneve0']
    assert len(geneve_obj.data['geneve']) == 11
    assert sorted(geneve_obj.data['geneve']) == sorted(['geneve', 'id', '10', 'remote', '192.168.43.254', 'dstport', '6081', 'noudpcsum', 'udp6zerocsumrx', 'addrgenmode', 'eui64'])

    tm0 = link_info["tm0"]
    # import pdb;pdb.set_trace()
    assert tm0["mac"] == "d4:f5:ef:01:1a:3c"
    assert tm0["flags"] == ['BROADCAST', 'MULTICAST', 'UP', 'LOWER_UP']
    assert tm0["type"] == "ether"
    assert tm0["mtu"] == 1500
    assert tm0['promiscuity'] == '1'
    assert tm0["rx_packets"] == 11784

    ens1f0 = link_info_all_4["ens1f0"]
    assert ens1f0["vf_enabled"] is False
    ens1f1 = link_info_all_4["ens1f1"]
    assert ens1f1["vf_enabled"] is True

    ens1 = link_info_all_5["ens1"]
    assert ens1["qlen"] == 1000


IP_ROUTE_SHOW_TABLE_ALL_TEST = """
throw 30.142.64.0/26  table red_mgmt
default via 30.142.64.1 dev bond0.400  table red_mgmt
throw 30.142.34.0/26  table red_storage
default via 30.142.34.1 dev bond0.300  table red_storage
30.0.0.0/8 dev notExist proto kernel scope link src 30.0.0.1
30.142.34.0/26 dev bond0.300  proto kernel  scope link  src 30.142.34.5
30.142.64.0/26 dev bond0.400  proto kernel  scope link  src 30.142.64.9
169.254.0.0/16 dev bond0  scope link  metric 1012
169.254.0.0/16 dev bond0.300  scope link  metric 1013
169.254.0.0/16 dev bond0.400  scope link  metric 1014
169.254.0.0/16 dev bond0.700  scope link  metric 1015
default via 30.142.64.1 dev bond0.400
broadcast 127.255.255.255 dev lo  table local  proto kernel  scope link  src 127.0.0.1
local 30.142.64.9 dev bond0.400  table local  proto kernel  scope host  src 30.142.64.9
""".strip()

IP_ROUTE_SHOW_TABLE_ALL_TEST_2 = """
default via 192.168.23.250 dev enp0s25
10.0.0.0/8 via 10.64.54.1 dev tun0  proto static  metric 50
10.64.54.0/23 dev tun0  proto kernel  scope link  src 10.64.54.44  metric 50
66.187.239.220 via 192.168.23.250 dev enp0s25  proto static
192.168.23.0/24 dev enp0s25  proto kernel  scope link  src 192.168.23.37
192.168.122.0/24 dev virbr0  proto kernel  scope link  src 192.168.122.1
broadcast 10.64.54.0 dev tun0  table local  proto kernel  scope link  src 10.64.54.44
local 10.64.54.44 dev tun0  table local  proto kernel  scope host  src 10.64.54.44
broadcast 10.64.55.255 dev tun0  table local  proto kernel  scope link  src 10.64.54.44
broadcast 127.0.0.0 dev lo  table local  proto kernel  scope link  src 127.0.0.1
local 127.0.0.0/8 dev lo  table local  proto kernel  scope host  src 127.0.0.1
local 127.0.0.1 dev lo  table local  proto kernel  scope host  src 127.0.0.1
broadcast 127.255.255.255 dev lo  table local  proto kernel  scope link  src 127.0.0.1
broadcast 192.168.23.0 dev enp0s25  table local  proto kernel  scope link  src 192.168.23.37
local 192.168.23.37 dev enp0s25  table local  proto kernel  scope host  src 192.168.23.37
broadcast 192.168.23.255 dev enp0s25  table local  proto kernel  scope link  src 192.168.23.37
broadcast 192.168.122.0 dev virbr0  table local  proto kernel  scope link  src 192.168.122.1
local 192.168.122.1 dev virbr0  table local  proto kernel  scope host  src 192.168.122.1
broadcast 192.168.122.255 dev virbr0  table local  proto kernel  scope link  src 192.168.122.1
unreachable ::/96 dev lo  metric 1024  error -101
unreachable ::ffff:0.0.0.0/96 dev lo  metric 1024  error -101
2001:708:40:2001:a822:baff:fec4:2428 via fe80::a96:d7ff:fe38:d757 dev enp0s25  metric 0
    cache  mtu 1492 hoplimit 255
2001:44b8:1110:f800::/64 dev enp0s25  proto kernel  metric 256  expires 6403sec mtu 1492
unreachable 2002:a00::/24 dev lo  metric 1024  error -101
unreachable 2002:7f00::/24 dev lo  metric 1024  error -101
unreachable 2002:a9fe::/32 dev lo  metric 1024  error -101
unreachable 2002:ac10::/28 dev lo  metric 1024  error -101
unreachable 2002:c0a8::/32 dev lo  metric 1024  error -101
unreachable 2002:e000::/19 dev lo  metric 1024  error -101
2404:6800:4006:802::200e via fe80::a96:d7ff:fe38:d757 dev enp0s25  metric 0
    cache  mtu 1492 hoplimit 255
2404:6800:4006:803::2003 via fe80::a96:d7ff:fe38:d757 dev enp0s25  metric 0
    cache  mtu 1492 hoplimit 255
2404:6800:4008:c00::bc via fe80::a96:d7ff:fe38:d757 dev enp0s25  metric 0
    cache  mtu 1492 hoplimit 255
2404:6800:4008:c02::bd via fe80::a96:d7ff:fe38:d757 dev enp0s25  metric 0
    cache  mtu 1492 hoplimit 255
2404:6800:4008:c06::bd via fe80::a96:d7ff:fe38:d757 dev enp0s25  metric 0
    cache  mtu 1492 hoplimit 255
2a00:1450:400e:800::2003 via fe80::a96:d7ff:fe38:d757 dev enp0s25  metric 0
    cache  mtu 1492 hoplimit 255
unreachable 3ffe:ffff::/32 dev lo  metric 1024  error -101
fe80::/64 dev enp0s25  proto kernel  metric 256  mtu 1492
default via fe80::a96:d7ff:fe38:d757 dev enp0s25  proto ra  metric 1024  expires 1568sec mtu 1492 hoplimit 255
unreachable default dev lo  table unspec  proto kernel  metric 4294967295  error -101
local ::1 dev lo  table local  proto none  metric 0
local 2001:44b8:1110:f800:56ee:75ff:fe1c:5901 dev lo  table local  proto none  metric 0
local fe80::56ee:75ff:fe1c:5901 dev lo  table local  proto none  metric 0
multicast 2001:dbc::/64 dev enp0s25
ff02::fb dev enp0s25  metric 0
    cache  mtu 1492
ff00::/8 dev enp0s25  table local  metric 256  mtu 1492
unreachable default dev lo  table unspec  proto kernel  metric 4294967295  error -101

""".strip()

IP_ROUTE_CANNOT_DETECT_THESE_ERRORS_YET = """
/usr/sbin/ip: bad command or file name
""".strip()

IP_ROUTE_SHOW_TABLE_ALL_TEST_BAD = """
ff00::/16
ff01::/16 dev
ff02::/16 dev enp0s25 metric
""".strip()


def test_ip_route():
    context = context_wrap(IP_ROUTE_SHOW_TABLE_ALL_TEST)
    d = ip.RouteDevices(context)
    assert len(d.data) == 6
    assert d["30.142.34.0/26"][0].dev == "bond0.300"
    assert d["30.142.64.0/26"][0].dev == "bond0.400"
    assert d["169.254.0.0/16"][0].dev == "bond0"
    assert d.ifaces(None) is None
    assert d.ifaces("30.142.34.1")[0] == "bond0.300"
    assert d.ifaces("30.142.64.1")[0] == "bond0.400"
    assert d.ifaces("169.254.0.1")[0] == "bond0"
    assert d.ifaces("30.0.0.1")[0] == "notExist"
    assert d.ifaces("192.168.0.1")[0] == "bond0.400"
    assert len(d.data["default"]) == 1
    d_devices = d.by_device['bond0.300']
    for idx in d_devices:
        if idx and getattr(idx, 'prefix') and getattr(idx, 'prefix') == '30.142.34.0/26':
            assert getattr(idx, 'src') == '30.142.34.5'
    d_prefix = d.by_prefix['default']
    assert getattr(d_prefix[0], 'via') == '30.142.64.1'
    d_prefix = d.by_prefix['30.142.34.0/26']
    assert getattr(d_prefix[0], 'scope') == 'link'
    d_table = d.by_table['None']
    for idx in d_table:
        if idx and getattr(idx, 'prefix') and getattr(idx, 'prefix') == "30.0.0.0/8":
            assert getattr(idx, 'proto') == 'kernel'
    # order is not deterministic
    # assert getattr(d.by_type['None'][3], 'netmask') == 8


def test_ip_route_2():
    context = context_wrap(IP_ROUTE_SHOW_TABLE_ALL_TEST_2)
    context.path = "sos_commands/networking/ip_route_show_all"
    tbl = ip.RouteDevices(context)

    assert len(tbl.data) == 26

    # Ignored route types:
    assert 'broadcast' not in tbl
    assert 'local' not in tbl
    assert 'unreachable' not in tbl
    assert 'throw' not in tbl
    assert 'prohibit' not in tbl
    assert 'blackhole' not in tbl
    assert 'nat' not in tbl

    # Standard route types:
    assert 'default' in tbl

    # Assert existence of the standard route attributes for all route
    # objects, even though the attribute itself may be None
    for prefix in tbl.data.keys():
        for route in tbl[prefix]:
            for attr in ['via', 'dev', 'type', 'netmask', 'prefix', 'table']:
                assert hasattr(route, attr)

    # Route object checks
    # 10.0.0.0/8 via 10.64.54.1 dev tun0  proto static  metric 50
    assert '10.0.0.0/8' in tbl
    r1 = tbl['10.0.0.0/8']
    assert len(r1) == 1
    r1a = r1[0]
    assert hasattr(r1a, 'prefix')
    assert r1a.prefix == '10.0.0.0/8'
    assert hasattr(r1a, 'via')
    assert r1a.via == '10.64.54.1'
    assert hasattr(r1a, 'dev')
    assert r1a.dev == 'tun0'
    assert hasattr(r1a, 'proto')
    assert r1a.proto == 'static'
    assert hasattr(r1a, 'metric')
    assert r1a.metric == '50'

    # 10.64.54.0/23 dev tun0  proto kernel  scope link  src 10.64.54.44  metric 50
    assert '10.64.54.0/23' in tbl
    r2 = tbl['10.64.54.0/23']
    assert len(r2) == 1
    r2a = r2[0]
    assert hasattr(r2a, 'prefix')
    assert r2a.prefix == '10.64.54.0/23'
    assert hasattr(r2a, 'via')
    assert r2a.via is None
    assert hasattr(r2a, 'dev')
    assert r2a.dev == 'tun0'
    assert hasattr(r2a, 'proto')
    assert r2a.proto == 'kernel'
    assert hasattr(r2a, 'scope')
    assert r2a.scope == 'link'
    assert hasattr(r2a, 'src')
    assert r2a.src == '10.64.54.44'
    assert hasattr(r2a, 'metric')
    assert r2a.metric == '50'

    # 2001:708:40:2001:a822:baff:fec4:2428 via fe80::a96:d7ff:fe38:d757 dev enp0s25  metric 0
    #     cache  mtu 1492 hoplimit 255
    assert '2001:708:40:2001:a822:baff:fec4:2428' in tbl
    r3 = tbl['2001:708:40:2001:a822:baff:fec4:2428']
    assert len(r3) == 1
    r3a = r3[0]
    assert hasattr(r3a, 'prefix')
    assert r3a.prefix == '2001:708:40:2001:a822:baff:fec4:2428'
    assert hasattr(r3a, 'via')
    assert r3a.via == 'fe80::a96:d7ff:fe38:d757'
    assert hasattr(r3a, 'dev')
    assert r3a.dev == 'enp0s25'
    assert hasattr(r3a, 'metric')
    assert r3a.metric == '0'
    assert hasattr(r3a, 'cache')
    assert r3a.cache
    assert hasattr(r3a, 'mtu')
    assert r3a.mtu == '1492'
    assert hasattr(r3a, 'hoplimit')
    assert r3a.hoplimit == '255'

    # by_prefix checks
    prefixes = tbl.by_prefix
    for prefix, route in prefixes.items():
        assert route[0] in tbl.by_device[route[0].dev]

    # by_device checks
    devices = tbl.by_device
    assert 'tun0' in devices
    assert 'enp0s25' in devices
    assert 'virbr0' in devices
    assert len(devices) == 4
    assert len(devices['enp0s25']) == 17

    # The order of these data structures is non-deterministic
    # Should be in order of reading, so find them in order by name
    # assert devices['enp0s25'][0] == tbl['2001:44b8:1110:f800::/64'][0]
    # assert devices['enp0s25'][1] == tbl['fe80::/64'][0]
    # assert devices['enp0s25'][2] == tbl['2001:708:40:2001:a822:baff:fec4:2428'][0]
    # Default routes accumulate in the same order?
    # assert devices['enp0s25'][3] == tbl['default'][0]
    # assert devices['enp0s25'][4] == tbl['default'][1]
    # assert devices['enp0s25'][5] == tbl['2404:6800:4008:c02::bd'][0]
    # assert devices['enp0s25'][6] == tbl['2a00:1450:400e:800::2003'][0]
    # assert devices['enp0s25'][7] == tbl['66.187.239.220'][0]
    # assert devices['enp0s25'][8] == tbl['2404:6800:4006:803::2003'][0]
    # assert devices['enp0s25'][9] == tbl['192.168.23.0/24'][0]
    # assert devices['enp0s25'][10] == tbl['2404:6800:4006:802::200e'][0]
    # assert devices['enp0s25'][11] == tbl['2404:6800:4008:c00::bc'][0]
    # assert devices['enp0s25'][12] == tbl['2404:6800:4008:c06::bd'][0]
    # assert devices['enp0s25'][13] == tbl['2001:dbc::/64'][0]
    # assert devices['enp0s25'][14] == tbl['ff02::fb'][0]
    # assert devices['enp0s25'][15] == tbl['ff00::/8'][0]

    # by_type checks
    types = tbl.by_type
    assert sorted(types.keys()) == sorted(['None', 'multicast', 'local'])
    # As we have added `local` in the SAVED_TYPES number of local rules which was
    # treaded as None now treating as a local so this number is changed
    assert len(types['None']) == 18
    for entry in types['None']:
        assert entry in tbl[entry.prefix]

    # by_table checks
    tables = tbl.by_table
    assert sorted(tables.keys()) == sorted(['None', 'local'])
    assert len(tables['None']) == 18
    # Something tells me that out of
    assert len(tables['local']) == 9
    for local in tables['local']:
        if local.dev == 'enp0s25' and local.prefix == 'ff00::/8':
            assert local.via is None
            assert local.metric == '256'
            assert local.prefix == 'ff00::/8'
            assert local.dev == 'enp0s25'
            assert local.mtu == '1492'
            assert local.netmask == 8
            assert local.table == 'local'
            assert local.type is None
        if local.dev == 'lo' and local.prefix == '2001:44b8:1110:f800:56ee:75ff:fe1c:5901':
            assert local.via is None
            assert local.metric == '0'
            assert local.table == 'local'
            assert local.type == 'local'
            assert local.dev == 'lo'


def test_bad_ip_routes():
    context = context_wrap(IP_ROUTE_SHOW_TABLE_ALL_TEST_BAD)
    context.path = "sos_commands/networking/ip_route_show_all"
    tbl = ip.RouteDevices(context)

    # As long as the line has a CIDR-net or address, the Route object is
    # instantiated but most of its fields are None
    assert len(tbl.data) == 3

    assert 'ff00::/16' in tbl
    r0a = tbl['ff00::/16'][0]
    for attr in ['via', 'dev', 'type', 'table']:
        assert hasattr(r0a, attr)
        assert getattr(r0a, attr) is None
    assert hasattr(r0a, 'prefix')
    assert r0a.prefix == 'ff00::/16'
    assert hasattr(r0a, 'netmask')
    assert r0a.netmask == 16

    assert 'ff01::/16' in tbl
    r1a = tbl['ff01::/16'][0]
    for attr in ['via', 'dev', 'type', 'table']:
        assert hasattr(r1a, attr)
        assert getattr(r1a, attr) is None
    assert hasattr(r1a, 'prefix')
    assert r1a.prefix == 'ff01::/16'
    assert hasattr(r1a, 'netmask')
    assert r1a.netmask == 16

    assert 'ff02::/16' in tbl
    r2a = tbl['ff02::/16'][0]
    for attr in ['via', 'type', 'table']:
        assert hasattr(r2a, attr)
        assert getattr(r2a, attr) is None
    assert hasattr(r2a, 'prefix')
    assert r2a.prefix == 'ff02::/16'
    assert hasattr(r2a, 'netmask')
    assert r2a.netmask == 16
    assert hasattr(r2a, 'dev')
    assert r2a.dev == 'enp0s25'
    assert not hasattr(r2a, 'metric')

    # Also a good chance to test the failure to find an interface for a
    # routed address, since we don't have a default route defined.
    assert tbl.ifaces('192.168.0.1') is None


IPV4_NEIGH_CONTEXT = """
172.17.42.10 dev lo lladdr 00:00:00:00:00:00 PERMANENT
172.17.42.11 dev lo lladdr 00:00:00:00:00:01 NOARP
172.17.42.12 dev lo lladdr 00:00:00:00:00:02 REACHABLE
172.17.42.13 dev lo lladdr 00:00:00:00:00:03 STALE
172.17.42.14 dev lo lladdr 00:00:00:00:00:04 DELAY
172.17.42.15 dev lo lladdr 00:00:00:00:00:05 FAILED
172.17.0.18 FAILED
172.17.0.19 dev docker0  FAILED
"""


def test_ipv4_neigh():
    result = ip.Ipv4Neigh(context_wrap(IPV4_NEIGH_CONTEXT))
    assert '172.17.0.18' in result
    assert len(result.data) == 8

    # Test all unreachability detection states
    assert result["172.17.42.10"] == {
        "dev": "lo", "lladdr": "00:00:00:00:00:00", "nud": "PERMANENT",
        'addr': ipaddress.ip_address(u'172.17.42.10')
    }
    assert result["172.17.42.11"] == {
        "dev": "lo", "lladdr": "00:00:00:00:00:01", "nud": "NOARP",
        'addr': ipaddress.ip_address(u'172.17.42.11')
    }
    assert result["172.17.42.12"] == {
        "dev": "lo", "lladdr": "00:00:00:00:00:02", "nud": "REACHABLE",
        'addr': ipaddress.ip_address(u'172.17.42.12')
    }
    assert result["172.17.42.13"] == {
        "dev": "lo", "lladdr": "00:00:00:00:00:03", "nud": "STALE",
        'addr': ipaddress.ip_address(u'172.17.42.13')
    }
    assert result["172.17.42.14"] == {
        "dev": "lo", "lladdr": "00:00:00:00:00:04", "nud": "DELAY",
        'addr': ipaddress.ip_address(u'172.17.42.14')
    }
    assert result["172.17.42.15"] == {
        "dev": "lo", "lladdr": "00:00:00:00:00:05", "nud": "FAILED",
        'addr': ipaddress.ip_address(u'172.17.42.15')
    }
    assert result["172.17.0.18"] == {
        "nud": "FAILED", 'addr': ipaddress.ip_address(u'172.17.0.18')
    }
    assert result["172.17.0.19"] == {
        "dev": "docker0", "nud": "FAILED",
        'addr': ipaddress.ip_address(u'172.17.0.19')
    }


BAD_NEIGH_CONTEXT = """
/usr/sbin/ip: bad command or file name
172.17.0
172.17.0.19
172.17.42.1 dev
172.17.42.2 dev lo
172.17.42.2 dev lo lladdr
"""


def test_bad_neigh():
    result = ip.Ipv4Neigh(context_wrap(BAD_NEIGH_CONTEXT))
    assert len(result.data) == 0
    # Check unparsed_lines?
    assert len(result.unparsed_lines) == 6


IPV6_NEIGH_CONTEXT = """
ff02::16 dev vlinuxbr lladdr 33:33:00:00:00:16 NOARP
ff02::1:ffea:2c00 dev tun0 lladdr 33:33:ff:ea:2c:00 NOARP
"""


def test_ipv6_neigh():
    result = ip.Ipv6Neigh(context_wrap(IPV6_NEIGH_CONTEXT))
    assert len(result.data) == 2
    assert len(result["ff02::16"]) == 4
    assert len(result["ff02::1:ffea:2c00"]) == 4
    assert result["ff02::16"] == {
        "dev": "vlinuxbr", "nud": "NOARP", "lladdr": "33:33:00:00:00:16",
        'addr': ipaddress.ip_address(u'ff02::16')
    }
    assert result["ff02::1:ffea:2c00"] == {
        "dev": "tun0", "nud": "NOARP", "lladdr": "33:33:ff:ea:2c:00",
        'addr': ipaddress.ip_address(u'ff02::1:ffea:2c00')
    }


IP_NEIGH_SHOW = """
2a04:9a00:1:1:ec4:7aff:febb:d3ca dev bond0.104 lladdr 0c:c4:7a:bb:d3:ca REACHABLE
fe80::22a:6aff:fe65:c3c2 dev bond0.104 lladdr 00:2a:6a:65:c3:c2 router STALE
2a04:9a00:1:1:ec4:7aff:febd:50b2 dev bond0.104 lladdr 0c:c4:7a:bd:50:b2 REACHABLE
2a04:9a00:1:1:ec4:7aff:fe1e:390e dev bond0.104 lladdr 0c:c4:7a:1e:39:0e REACHABLE
2a04:9a00:1:1:ec4:7aff:fe1f:45a dev bond0.104 lladdr 0c:c4:7a:1f:04:5a REACHABLE
fe80::ec4:7aff:fe1f:808 dev bond0.104 lladdr 0c:c4:7a:1f:08:08 STALE
2a04:9a00:1:1:ec4:7aff:febb:d1fe dev bond0.104 lladdr 0c:c4:7a:bb:d1:fe REACHABLE
10.101.3.27 dev bond0.101 lladdr 00:90:fa:7f:b0:78 STALE
10.101.5.252 dev bond0.102 lladdr 00:2a:6a:65:c5:42 STALE
10.101.5.27 dev bond0.102 lladdr 0c:c4:7a:8f:44:02 DELAY
10.101.3.252 dev bond0.101 lladdr 00:2a:6a:65:c5:42 STALE
10.101.3.21 dev bond0.101 lladdr 00:90:fa:7f:ad:d8 REACHABLE
10.101.5.71 dev bond0.102  INCOMPLETE
10.101.5.17 dev bond0.102  INCOMPLETE
""".strip()


def test_ip_neigh_show():
    result = ip.IpNeighShow(context_wrap(IP_NEIGH_SHOW))
    assert len(result.data) == 13
    assert result['2a04:9a00:1:1:ec4:7aff:febb:d3ca']['nud'] == 'REACHABLE'
    assert len(result['2a04:9a00:1:1:ec4:7aff:febb:d1fe']) == 4
    assert len(result['10.101.5.252']) == 4
    assert result['10.101.5.17']['nud'] == 'INCOMPLETE'
    assert result['10.101.5.71']['nud'] == 'INCOMPLETE'
    assert len(result['10.101.5.71']) == 3
    assert len(result['10.101.5.17']) == 3


HOSTNAME_I = """
10.230.230.220 10.230.0.1 1000:52:0:4996:546f:cbff:fe47:00
""".strip()


def test_hostname_I_addresses():
    result = ip.IPs(context_wrap(HOSTNAME_I))
    assert result.ipv4_addresses == ['10.230.230.220', '10.230.0.1']



"""
EtcMachineId - file ``/etc/machine-id``
=======================================
"""
import uuid

from insights.core import Parser
from insights.core.exceptions import SkipComponent
from insights.core.plugins import parser
from insights.parsers import get_active_lines
from insights.specs import Specs


@parser(Specs.etc_machine_id)
class EtcMachineId(Parser):
    """
    Class for parsing the file ``/etc/machine-id``

    Sample file content::

        4f3fbfda59ff4aea969255a73342d439

    Examples:
        >>> type(machine_id)
        <class 'insights.parsers.etc_machine_id.EtcMachineId'>
        >>> machine_id.machine_id
        '4f3fbfda59ff4aea969255a73342d439'
        >>> machine_id.uuid
        '4f3fbfda-59ff-4aea-9692-55a73342d439'
    """

    def parse_content(self, content):
        content = get_active_lines(content)
        if not content or len(content) != 1:
            raise SkipComponent('Invalid Content')
        self.id = self.machine_id = content[0]
        self.uuid = str(uuid.UUID(self.id))



import pytest

from insights.plugins.ps_rule_fakes import (psaux_no_filter,
                                            psauxww_ds_filter,
                                            psalxwww_parser_filter)
from insights.plugins.rule_with_combiner_and_parser_parser_filtered import parser_is_filtered
from insights.plugins.rule_with_combiner_and_parser_parser_not_filtered import parser_is_not_filtered
from insights.plugins.rule_with_combiner_only import parser_is_not_used
from insights.specs import Specs
from insights.tests import InputData, run_test


def test_run_test_missing_filters_exception():
    """
    The rule underlying datasource requires a filter,
    an exception should be raised because filter was not
    added in the rule module.
    """
    input_data = InputData("fake_input")
    input_data.add(Specs.ps_aux, "FAKE_CONTENT")
    with pytest.raises(Exception):
        run_test(psaux_no_filter, input_data, None)


def test_run_test_no_missing_filters_using_datasource():
    """
    Required filter was added directly to the datasouce,
    ``run_test`` should complete without any exceptions.
    """
    input_data = InputData("fake_input")
    input_data.add(Specs.ps_auxww, "FAKE_CONTENT")
    result = run_test(psauxww_ds_filter, input_data, None)
    assert result


def test_run_test_no_missing_filters_using_parser():
    """
    Required filter was added to using the parser,
    ``run_test`` should complete without any exceptions.
    """
    input_data = InputData("fake_input")
    input_data.add(Specs.ps_alxwww, "FAKE_CONTENT")
    result = run_test(psalxwww_parser_filter, input_data, None)
    assert result


def test_run_test_parser_is_not_used():
    """
    1. The rule uses one Combiner that depends upon a filterable Parser.
    2. The rule does NOT depend upon (import) the filterable Parser directly.
    3. The filterable Parser is NOT filter in the rule as well.
    """
    input_data = InputData("fake_input")
    input_data.add(Specs.dmesg, "FAKE_CONTENT")
    result = run_test(parser_is_not_used, input_data)
    # No Exception raised
    assert result


def test_run_test_parser_is_filtered():
    """
    1. The rule uses one Combiner that depends upon a filterable Parser.
    2. The rule does also depend upon (import) the filterable Parser directly.
    3. The filterable Parser is FILTERED in the rule.
    """
    input_data = InputData("fake_input")
    input_data.add(Specs.dmesg, "FAKE_CONTENT")
    result = run_test(parser_is_filtered, input_data)
    # No Exception raised
    assert result


def test_run_test_parser_is_not_filtered():
    """
    1. The rule uses one Combiner that depends upon a filterable Parser.
    2. The rule does also depend upon (import) the filterable Parser.
    3. The filterable Parser is NOT filtered in the rule.
    """
    input_data = InputData("fake_input")
    input_data.add(Specs.dmesg, "FAKE_CONTENT")
    with pytest.raises(Exception) as ex:
        run_test(parser_is_not_filtered, input_data, None)
    assert 'must add filters to' in str(ex)



import json
import pytest

from mock.mock import patch
from collections import defaultdict

from insights.client.config import InsightsConfig
from insights.core import filters
from insights.core.exceptions import SkipComponent
from insights.core.spec_factory import DatasourceProvider
from insights.specs import Specs
from insights.specs.datasources.machine_ids import dup_machine_id_info
from insights.tests import context_wrap


class MockInsightsConnection(object):
    def __init__(self, config=None):
        self.config = config
        self.base_url = ''
        self.inventory_url = ''

    def get(self, url):
        if url.endswith('45678'):
            content = json.dumps(
                {
                    'total': 1,
                    "count": 1,
                    "page": 1,
                    "per_page": 50,
                    "results": [
                        {
                            "insights_id": "95160ae3-1ee1-40e1-9666-316dbb9270dd",
                            "subscription_manager_id": "a5a71082-1d09-4fc8-bffc-ad326e24df6a",
                            "satellite_id": None,
                            "ip_addresses": [
                                "10.0.222.82"
                            ],
                            "fqdn": "hostname1.compute.internal",

                        }
                    ]
                }
            )
        elif url.endswith('deff'):
            content = json.dumps(
                {
                    'total': 2,
                    "count": 2,
                    "page": 1,
                    "per_page": 50,
                    "results": [
                        {
                            "insights_id": "95160ae3-1ee1-40e1-9666-316dbb927ff1",
                            "subscription_manager_id": "sfwerwfsf-1d09-4fc8-bffc-sfxcsfsf",
                            "satellite_id": None,
                            "ip_addresses": [
                                "10.0.222.82"
                            ],
                            "fqdn": "hostname1.compute.internal",

                        },
                        {
                            "insights_id": "95160ae3-1ee1-40e1-9666-316dbb927ff1",
                            "subscription_manager_id": "sfweswersf-1d09-4fc8-bffc-sfxcssdf",
                            "satellite_id": None,
                            "ip_addresses": [
                                "10.0.222.83"
                            ],
                            "fqdn": "hostname2.compute.internal",

                        }
                    ]
                }
            )
        elif url.endswith('wrong'):
            content = "wrong result"
        return Result(content)


class Result(object):
    def __init__(self, content):
        self.content = content


def setup_function(func):
    if func in [test_duplicate, test_non_duplicate, test_wrong_machine_id_content, test_machine_id_not_in_filters, test_api_result_not_in_json_format]:
        filters.add_filter(Specs.duplicate_machine_id, ["dc194312-8cdd-4e75-8cf1-2094bfsfsdeff"])
        filters.add_filter(Specs.duplicate_machine_id, ["dc194312-8cdd-4e75-8cf1-2094bf45678"])
        filters.add_filter(Specs.duplicate_machine_id, ["dc194312-8cdd-4e75-8cf1-2094bfwrong"])
    if func is test_module_filters_empty:
        filters.add_filter(Specs.duplicate_machine_id, [])


def teardown_function(func):
    filters._CACHE = {}
    filters.FILTERS = defaultdict(set)


@patch('insights.specs.datasources.machine_ids.get_connection',
       return_value=MockInsightsConnection())
def test_duplicate(conn):
    broker = {
        Specs.machine_id: context_wrap(lines=['dc194312-8cdd-4e75-8cf1-2094bfsfsdeff']),
        'client_config': InsightsConfig(legacy_upload=False)
    }
    result = dup_machine_id_info(broker)
    expected = DatasourceProvider(content=["dc194312-8cdd-4e75-8cf1-2094bfsfsdeff hostname1.compute.internal,hostname2.compute.internal"], relative_path='insights_commands/duplicate_machine_id_info')
    assert expected.content == result.content
    assert expected.relative_path == result.relative_path

    broker = {
        Specs.machine_id: context_wrap(lines=['dc194312-8cdd-4e75-8cf1-2094bfsfsdeff']),
        'client_config': InsightsConfig(legacy_upload=True)
    }
    result = dup_machine_id_info(broker)
    expected = DatasourceProvider(content=["dc194312-8cdd-4e75-8cf1-2094bfsfsdeff hostname1.compute.internal,hostname2.compute.internal"], relative_path='insights_commands/duplicate_machine_id_info')
    assert expected.content == result.content
    assert expected.relative_path == result.relative_path


@patch('insights.specs.datasources.machine_ids.get_connection',
       return_value=MockInsightsConnection())
def test_non_duplicate(conn):
    broker = {
        Specs.machine_id: context_wrap(lines=['dc194312-8cdd-4e75-8cf1-2094bf45678']),
        'client_config': InsightsConfig(legacy_upload=False)
    }
    with pytest.raises(SkipComponent):
        dup_machine_id_info(broker)


@patch('insights.specs.datasources.machine_ids.get_connection',
       return_value=MockInsightsConnection())
def test_module_filters_empty(conn):
    broker = {
        Specs.machine_id: context_wrap(lines=['dc194312-8cdd-4e75-8cf1-2094bfsfsdeff']),
        'client_config': InsightsConfig(legacy_upload=False)
    }
    with pytest.raises(SkipComponent):
        dup_machine_id_info(broker)


@patch('insights.specs.datasources.machine_ids.get_connection',
       return_value=MockInsightsConnection())
def test_wrong_machine_id_content(conn):
    broker = {
        Specs.machine_id: context_wrap(lines=['dc194312-8cdd-4e75-8cf1-2094bfsfsdeff', 'dc194312-8cdd-4e75-8cf1-2094bfsf45678']),
        'client_config': InsightsConfig(legacy_upload=False)
    }
    with pytest.raises(SkipComponent):
        dup_machine_id_info(broker)


@patch('insights.specs.datasources.machine_ids.get_connection',
       return_value=MockInsightsConnection())
def test_machine_id_not_in_filters(conn):
    broker = {
        Specs.machine_id: context_wrap(lines=['dc194312-8cdd-4e75-8cf1-2094bfsfsabc']),
        'client_config': InsightsConfig(legacy_upload=False)
    }
    with pytest.raises(SkipComponent):
        dup_machine_id_info(broker)


@patch('insights.specs.datasources.machine_ids.get_connection',
       return_value=MockInsightsConnection("wrong"))
def test_api_result_not_in_json_format(conn):
    broker = {
        Specs.machine_id: context_wrap(lines=['dc194312-8cdd-4e75-8cf1-2094bfwrong']),
        'client_config': InsightsConfig(legacy_upload=False)
    }
    with pytest.raises(SkipComponent):
        dup_machine_id_info(broker)



"""
RhospRelease - file ``/etc/rhosp-release``
==========================================

This module provides plugins access to file ``/etc/rhosp-release``

Typical content of file ``/etc/rhosp-release`` is::

    Red Hat OpenStack Platform release 14.0.0 RC (Rocky)

This module parses the file content and stores data in the dict ``self.release``
with keys ``product``, ``version``, and ``code_name``.

Examples:
    >>> release.product
    'Red Hat OpenStack Platform'
    >>> release.version
    '14.0.0'
    >>> release.code_name
    'Rocky'
"""
from insights import Parser, parser
from insights.specs import Specs


@parser(Specs.rhosp_release)
class RhospRelease(Parser):
    """Parses the content of file ``/etc/rhosp-release``."""

    def parse_content(self, content):
        product, _, version_name = [v.strip() for v in content[0].partition("release")]
        version_name_split = [v.strip() for v in version_name.split(None, 1)]
        self.release = {
            "product": product,
            "version": version_name_split[0],
            "code_name": version_name_split[1].split()[-1].strip("()")
        }

    @property
    def version(self):
        """string: Version of RHOSP."""
        return self.release["version"]

    @property
    def product(self):
        """string: Product full name."""
        return self.release["product"]

    @property
    def code_name(self):
        """string: Release code name."""
        return self.release["code_name"]



from datetime import datetime
from insights.parsers.up2date_log import Up2dateLog
from insights.tests import context_wrap
import doctest
from insights.parsers import up2date_log

LOG1 = """
[Thu Feb  1 10:40:13 2018] rhn_register ERROR: can not find RHNS CA file: /usr/share/rhn/RHN-ORG-TRUSTED-SSL-CERT
[Thu Feb  1 10:40:22 2018] rhn_register ERROR: can not find RHNS CA file: /usr/share/rhn/RHN-ORG-TRUSTED-SSL-CERT
[Thu Feb  1 10:40:22 2018] rhn_register
Traceback (most recent call last):
  File "/usr/sbin/rhn_register", line 74, in <module>
    app.run()
  File "/usr/share/rhn/up2date_client/rhncli.py", line 96, in run
    sys.exit(self.main() or 0)
  File "/usr/sbin/rhn_register", line 60, in main
    if not up2dateAuth.getLoginInfo():
  File "/usr/share/rhn/up2date_client/up2dateAuth.py", line 228, in getLoginInfo
    login(timeout=timeout)
  File "/usr/share/rhn/up2date_client/up2dateAuth.py", line 179, in login
    server = rhnserver.RhnServer(timeout=timeout)
  File "/usr/share/rhn/up2date_client/rhnserver.py", line 172, in __init__
    timeout=timeout)
  File "/usr/share/rhn/up2date_client/rpcServer.py", line 172, in getServer
    raise up2dateErrors.SSLCertificateFileNotFound(msg)
<class 'up2date_client.up2dateErrors.SSLCertificateFileNotFound'>: ERROR: can not find RHNS CA file: /usr/share/rhn/RHN-ORG-TRUSTED-SSL-CERT
""".strip()

LOG2 = """
[Thu Feb  1 02:46:25 2018] rhn_register updateLoginInfo() login info
[Thu Feb  1 02:46:35 2018] rhn_register A socket error occurred: (-3, 'Temporary failure in name resolution'), attempt #1
[Thu Feb  1 02:46:40 2018] rhn_register A socket error occurred: (-3, 'Temporary failure in name resolution'), attempt #2
[Thu Feb  1 02:46:45 2018] rhn_register A socket error occurred: (-3, 'Temporary failure in name resolution'), attempt #3
[Thu Feb  1 02:46:50 2018] rhn_register A socket error occurred: (-3, 'Temporary failure in name resolution'), attempt #4
[Thu Feb  1 02:46:55 2018] rhn_register A socket error occurred: (-3, 'Temporary failure in name resolution'), attempt #5
"""


def test_up2date_log():
    ulog = Up2dateLog(context_wrap(LOG1))
    ern_list = ulog.get('ERROR')
    assert 3 == len(ern_list)
    assert ern_list[2]['raw_message'] == "<class 'up2date_client.up2dateErrors.SSLCertificateFileNotFound'>: ERROR: can not find RHNS CA file: /usr/share/rhn/RHN-ORG-TRUSTED-SSL-CERT"
    assert len(list(ulog.get_after(datetime(2018, 2, 1, 10, 40, 22)))) == 18

    ulog = Up2dateLog(context_wrap(LOG2))
    ern_list = ulog.get('Temporary failure in name resolution')
    assert 5 == len(ern_list)
    assert ern_list[0]['raw_message'] == "[Thu Feb  1 02:46:35 2018] rhn_register A socket error occurred: (-3, 'Temporary failure in name resolution'), attempt #1"
    assert len(list(ulog.get_after(datetime(2018, 2, 1, 2, 46, 45)))) == 3


def test_up2date_log_doc_examples():
    env = {
        'Up2dateLog': Up2dateLog,
        'ulog': Up2dateLog(context_wrap(LOG2)),
    }
    failed, total = doctest.testmod(up2date_log, globs=env)
    assert failed == 0



"""
NfnetLinkQueue - file ``/proc/net/netfilter/nfnetlink_queue``
=============================================================
"""
from insights.core import Parser
from insights.core.exceptions import ParseException, SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.nfnetlink_queue)
class NfnetLinkQueue(Parser, list):
    """
    Reads the ``/proc/net/netfilter/nfnetlink_queue`` file and creates a list of
    dictionaries, one dictionary per row in the file.

    The keys of the dictionary are (see
    https://home.regit.org/netfilter-en/using-nfqueue-and-libnetfilter_queue/):

    - ``queue_number``
    - ``peer_portid``: good chance it is process ID of software listening to
      the queue
    - ``queue_total``: current number of packets waiting in the queue
    - ``copy_mode``: 0 and 1 only message only provide meta data. If 2, the
      message provides a part of packet of size copy range.
    - ``copy_range``: length of packet data to put in message
    - ``queue_dropped``: number of packets dropped because queue was full
    - ``user_dropped``: number of packets dropped because netlink message
      could not be sent to userspace. If this counter is not zero, try
      to increase netlink buffer size. On the application side, you will
      see gap in packet id if netlink message are lost.
    - ``id_sequence``: packet id of last packet
    - The last field is always '1' and is ignored.

    Example Input::

        0  -4423     0 2 65535     0     0       22  1
        1  -4424     0 2 65535     0     0       27  1

    Examples:

        >>> type(nf)
        <class 'insights.parsers.nfnetlink_queue.NfnetLinkQueue'>
        >>> 'copy_mode' in nf.data[0]
        True
        >>> nf.data[0]['copy_mode']
        2
    """

    def parse_content(self, content):
        if not content:
            raise SkipComponent("Empty output.")

        data = []
        fields = ['queue_number', 'peer_portid', 'queue_total', 'copy_mode',
                  'copy_range', 'queue_dropped', 'user_dropped', 'id_sequence']

        for line in content:
            parts = [int(p.strip()) for p in line.split()]

            if len(parts) != 9:
                raise ParseException("Can't parse content because there isn't 9 columns provided.")

            data.append(dict(zip(fields, parts)))

        if not data:
            raise SkipComponent("No parsed data.")

        self.extend(data)

    @property
    def data(self):
        """
        Set data as property to keep compatibility
        """
        return self



"""
1. The rule uses one Combiner that depends upon a filterable Parser.
2. The rule does NOT depend upon (import) the filterable Parser directly.
3. The filterable Parser is NOT filtered in the rule as well.

=> Exception is NOT expected
"""
from insights import rule, make_pass
from insights.combiners.os_release import OSRelease


@rule(OSRelease)
def parser_is_not_used(osr):
    return make_pass("FAKE RESULT")



"""
tomcat_xml - XML files for Tomcat
=================================

Classes to parse Tomcat XML configuration files:

TomcatWebXml - files from ``/etc/tomcat*/web.xml`` and ``/conf/tomcat/tomcat*/web.xml``
---------------------------------------------------------------------------------------

TomcatServerXml - files from ``(tomcat base directory)/conf/server.xml`` or ``conf/tomcat/tomcat*/server.xml``
--------------------------------------------------------------------------------------------------------------

.. note::
    The tomcat XML files are found in the directory specified in the java
    commands

"""
from .. import parser, XMLParser
from insights.specs import Specs


@parser(Specs.tomcat_web_xml)
class TomcatWebXml(XMLParser):
    """
    Parse the `web.xml` of Tomcat.

    Currently it only stores the setting of `session-timeout`.

    Attributes:
        data (dict): special settings, e.g. ``session-timeout`` get from the
            xml file.

    Sample input::

        <?xml version="1.0" encoding="ISO-8859-1"?>
        <web-app xmlns="http://java.sun.com/xml/ns/javaee"
            xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
            xsi:schemaLocation="http://java.sun.com/xml/ns/javaee http://java.sun.com/xml/ns/javaee/web-app_2_5.xsd"
            version="2.5">

            <servlet>
                <servlet-name>default</servlet-name>
                <servlet-class>org.apache.catalina.servlets.DefaultServlet</servlet-class>
                <init-param>
                    <param-name>debug</param-name>
                    <param-value>0</param-value>
                </init-param>
                <init-param>
                    <param-name>listings</param-name>
                    <param-value>false</param-value>
                </init-param>
                <load-on-startup>1</load-on-startup>
            </servlet>

            <session-config>
                <session-timeout>30</session-timeout>
            </session-config>

            <welcome-file-list>
                <welcome-file>index.html</welcome-file>
                <welcome-file>index.htm</welcome-file>
                <welcome-file>index.jsp</welcome-file>
            </welcome-file-list>

        </web-app>

    Examples:
        >>> type(web_xml)
        <class 'insights.parsers.tomcat_xml.TomcatWebXml'>
        >>> web_xml.get('session-timeout') == 30
        True
    """

    def parse_dom(self):
        """
        Get the setting of 'session-timeout' from the parsed `Elements` in
        :attr:`data` and return.

        Returns:
            (dict): Currently only 'session-timeout' is added into the
            dictionary.  An empty dict, when 'session-timeout' setting cannot
            be found.
        """
        key_field = self.get_elements('.//session-timeout')
        field_text = key_field[0].text if key_field else None

        parsed_data = {}
        if field_text and field_text.isdigit():
            parsed_data['session-timeout'] = int(field_text)
        return parsed_data


@parser(Specs.tomcat_server_xml)
class TomcatServerXml(XMLParser):
    """
    Parse the `server.xml` of Tomcat.

    Sample input::

        <?xml version='1.0' encoding='utf-8'?>
        <Server port="8005" shutdown="SHUTDOWN">

          <Listener className="org.apache.catalina.core.AprLifecycleListener" SSLEngine="on" />
          <Listener className="org.apache.catalina.core.JasperListener" />
          <Listener className="org.apache.catalina.core.JreMemoryLeakPreventionListener" />
          <Listener className="org.apache.catalina.mbeans.GlobalResourcesLifecycleListener" />
        <Listener className="org.apache.catalina.core.ThreadLocalLeakPreventionListener" />
          <GlobalNamingResources>
            <Resource name="UserDatabase" auth="Container"
                      type="org.apache.catalina.UserDatabase"
                      description="User database that can be updated and saved"
                      factory="org.apache.catalina.users.MemoryUserDatabaseFactory"
                      pathname="conf/tomcat-users.xml" />
          </GlobalNamingResources>
          <Service name="Catalina">
            <Connector port="8080" protocol="HTTP/1.1"
                       connectionTimeout="20000"
                       redirectPort="8443" />
            <Connector port="8443" protocol="HTTP/1.1" SSLEnabled="true"
                       maxThreads="150" scheme="https" secure="true"
                       clientAuth="want"
                       sslProtocols="TLSv1.2,TLSv1.1,TLSv1"
                       keystoreFile="conf/keystore"
                       truststoreFile="conf/keystore"
                       keystorePass="oXQ8LfAGsf97KQxwwPta2X3vnUv7P5QM"
                       keystoreType="PKCS12"
                       ciphers="SSL_RSA_WITH_3DES_EDE_CBC_SHA,
                            TLS_RSA_WITH_AES_256_CBC_SHA,
                            TLS_ECDH_ECDSA_WITH_AES_128_CBC_SHA,
                            TLS_ECDH_ECDSA_WITH_AES_256_CBC_SHA,
                            TLS_ECDH_RSA_WITH_3DES_EDE_CBC_SHA,
                            TLS_ECDH_RSA_WITH_AES_128_CBC_SHA,
                            TLS_ECDH_RSA_WITH_AES_256_CBC_SHA,
                            TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA"
                       truststorePass="oXQ8LfAGsf97KQxwwPta2X3vnUv7P5QM" />

            <Connector port="8009" protocol="AJP/1.3" redirectPort="8443" />
            <Engine name="Catalina" defaultHost="localhost">
              <Realm className="org.apache.catalina.realm.UserDatabaseRealm"
                     resourceName="UserDatabase"/>
              <Host name="localhost"  appBase="webapps"
                    unpackWARs="true" autoDeploy="true"
                    xmlValidation="false" xmlNamespaceAware="false">
              </Host>
            </Engine>
          </Service>
        </Server>

    Examples:
        >>> type(server_xml)
        <class 'insights.parsers.tomcat_xml.TomcatServerXml'>
        >>> server_xml.file_path
        '/usr/share/tomcat/server.xml'
        >>> hosts = server_xml.get_elements(".//Service/Engine/Host")
        >>> len(hosts)
        1
        >>> hosts[0].get('name')
        'localhost'

    """
    pass



# -*- coding: UTF-8 -*-
import pytest

from datetime import datetime

from insights.core import LogFileOutput
from insights.core.exceptions import ParseException
from insights.tests import context_wrap


class FakeMessagesClass(LogFileOutput):
    time_format = '%b %d %H:%M:%S'

    def bad_get_after(self, timestamp, lines=None):
        self.time_format = '%q/%1 %z:%v:%j'
        return super(FakeMessagesClass, self).get_after(timestamp, lines)

    def superget(self, s):
        for line in self.lines:
            if s in line:
                parts = line.split(None, 6)
                yield {
                    'timestamp': ' '.join(parts[0:3]),
                    'hostname': parts[3],
                    'service': parts[4][:-1],  # strip colon
                    'message': parts[5]
                }

    def listget(self, s):
        # Silly function to split lines containing s.
        for line in self.lines:
            if s in line:
                yield s.split(None, 6)


MESSAGES = """
Mar 27 03:18:15 system rsyslogd: [origin software="rsyslogd" swVersion="5.8.10" x-pid="1870" x-info="http://www.rsyslog.com"] first
Mar 27 03:18:16 system rsyslogd-2177: imuxsock lost 141 messages from pid 55082 due to rate-limiting
Mar 27 03:18:19 system rsyslogd-2177: imuxsock begins to drop messages from pid 55082 due to rate-limiting
Mar 27 03:18:21 system pulp: pulp.server.db.connection:INFO: Attempting Database connection with seeds = localhost:27017
Mar 27 03:18:21 system pulp: pulp.server.db.connection:INFO: Connection Arguments: {'max_pool_size': 10}
Mar 27 03:18:21 system pulp: pulp.server.db.connection:INFO: Database connection established with: seeds = localhost:27017, name = pulp_database
Mar 27 03:18:22 system rsyslogd-2177: imuxsock lost 145 messages from pid 55082 due to rate-limiting
Mar 27 03:18:24 system puppet-master[48226]: Setting manifest is deprecated in puppet.conf. See http://links.puppetlabs.com/env-settings-deprecations
Mar 27 03:18:24 system puppet-master[48226]:    (at /usr/lib/ruby/site_ruby/1.8/puppet/settings.rb:1095:in `issue_deprecations')
Mar 27 03:18:24 system puppet-master[48226]: Setting modulepath is deprecated in puppet.conf. See http://links.puppetlabs.com/env-settings-deprecations
Mar 27 03:18:24 system puppet-master[48226]:    (at /usr/lib/ruby/site_ruby/1.8/puppet/settings.rb:1095:in `issue_deprecations')
Mar 27 03:18:24 system puppet-master[48226]: Setting config_version is deprecated in puppet.conf. See http://links.puppetlabs.com/env-settings-deprecations
Mar 27 03:18:24 system puppet-master[48226]:    (at /usr/lib/ruby/site_ruby/1.8/puppet/settings.rb:1095:in `issue_deprecations')
Mar 27 03:18:25 system rsyslogd-2177: imuxsock begins to drop messages from pid 55082 due to rate-limiting
Mar 27 03:39:43 system pulp: pulp.server.webservices.middleware.exception:ERROR:   File "/usr/lib/python2.6/site-packages/web/application.py", line 230, in handle
     return self._delegate(fn, self.fvars, args)
   File "/usr/lib/python2.6/site-packages/web/application.py", line 405, in _delegate
     return handle_class(f)
   File "/usr/lib/python2.6/site-packages/web/application.py", line 396, in handle_class
     return tocall(*args)
   File "/usr/lib/python2.6/site-packages/pulp/server/webservices/controllers/decorators.py", line 227, in _auth_decorator
     value = method(self, *args, **kwargs)
   File "/usr/lib/python2.6/site-packages/pulp/server/webservices/controllers/consumers.py", line 503, in GET
     profile = manager.get_profile(consumer_id, content_type)
   File "/usr/lib/python2.6/site-packages/pulp/server/managers/consumer/profile.py", line 120, in get_profile
     raise MissingResource(profile_id=profile_id)
MissingResource: Missing resource(s): profile_id={'content_type': u'rpm', 'consumer_id': u'1786cd7f-2ab2-4212-9798-c0a454e97900'}
Mar 27 03:39:43 system rsyslogd-2177: imuxsock begins to drop messages from pid 55082 due to rate-limiting
Mar 27 03:39:46 system rsyslogd-2177: imuxsock lost 165 messages from pid 55082 due to rate-limiting
Mar 27 03:39:49 system rsyslogd-2177: imuxsock begins to drop messages from pid 55082 due to rate-limiting
Mar 27 03:49:10 system pulp: pulp.server.webservices.middleware.exception:ERROR: Missing resource(s): profile_id={'content_type': u'rpm', 'consumer_id': u'79d5aed1-5631-4f40-b970-585ee974eb87'}
"""


def test_messages_scanners():
    # Messages that are present can be kept
    FakeMessagesClass.keep_scan('puppet_master_logs', ' puppet-master')
    # Messages that are absent still turn up as an empty list
    FakeMessagesClass.keep_scan('kernel_logs', ' kernel')
    # Token scan of something that's present
    FakeMessagesClass.token_scan('middleware_exception_present', 'pulp.server.webservices.middleware.exception')
    # Token scan of something that's absent
    FakeMessagesClass.token_scan('cron_present', 'CRONTAB')

    # Check that duplicate scanners raise a value exception
    with pytest.raises(ValueError) as exc:
        FakeMessagesClass.keep_scan('kernel_logs', ' kernel')
    assert "'kernel_logs' is already a registered scanner key" in str(exc)

    def count_lost_messages(self):
        imuxsock_lines = 0
        lost_messages = 0
        for line in self.lines:
            if 'imuxsock lost' in line:
                parts = line.split(None)
                if parts[7].isdigit():
                    imuxsock_lines += 1
                    lost_messages += int(parts[7])
        return "lost {msgs} messages in {lines} lines".format(msgs=lost_messages, lines=imuxsock_lines)
    FakeMessagesClass.scan('lost_messages', count_lost_messages)

    ctx = context_wrap(MESSAGES, path='/var/log/messages')
    log = FakeMessagesClass(ctx)
    assert hasattr(log, 'puppet_master_logs')
    assert hasattr(log, 'kernel_logs')
    assert hasattr(log, 'middleware_exception_present')
    assert hasattr(log, 'cron_present')
    assert hasattr(log, 'lost_messages')

    assert len(log.puppet_master_logs) == 6
    assert log.puppet_master_logs[0]['raw_message'] == 'Mar 27 03:18:24 system puppet-master[48226]: Setting manifest is deprecated in puppet.conf. See http://links.puppetlabs.com/env-settings-deprecations'
    assert log.kernel_logs == []
    assert log.middleware_exception_present
    assert not log.cron_present
    assert log.lost_messages == 'lost 451 messages in 3 lines'


def test_messages_scanners_list():
    # Messages that are present can be kept
    FakeMessagesClass.keep_scan('puppet_master_manifest_logs', ['puppet-master', 'manifest'])
    FakeMessagesClass.keep_scan('puppet_master_first', ['puppet-master', 'first'])
    # Messages that are present can be kept for any of the list
    FakeMessagesClass.keep_scan('puppet_master_first_any', ['puppet-master', 'first'], check=any)
    FakeMessagesClass.keep_scan('puppet_master_first_any_3', ['puppet-master', 'first'], num=3, check=any)
    # Token scan of something that's absent
    FakeMessagesClass.token_scan('error_missing', ['ERROR', 'Missing'])
    # Token scan of something that's absent
    FakeMessagesClass.token_scan('error_info', ['ERROR', 'info'])
    # Token scan of something that's absent for any of the list
    FakeMessagesClass.token_scan('error_info_any', ['ERROR', 'info'], check=any)
    # Get the last line
    FakeMessagesClass.last_scan('puppet_master_manifest_last', ['puppet-master', 'manifest'])
    # Get the last line for any of the list
    FakeMessagesClass.last_scan('puppet_master_first_last_any', ['puppet-master', 'first'], check=any)

    ctx = context_wrap(MESSAGES, path='/var/log/messages')
    log = FakeMessagesClass(ctx)

    assert hasattr(log, 'puppet_master_manifest_logs')
    assert len(log.puppet_master_manifest_logs) == 1
    assert hasattr(log, 'error_missing')
    assert log.error_missing

    assert hasattr(log, 'puppet_master_first')
    assert len(log.puppet_master_first) == 0
    assert hasattr(log, 'error_info')
    assert log.error_info is False

    assert hasattr(log, 'puppet_master_first_any')
    assert len(log.puppet_master_first_any) == 7
    assert hasattr(log, 'puppet_master_first_any_3')
    assert len(log.puppet_master_first_any_3) == 3
    assert hasattr(log, 'error_info_any')
    assert log.error_info_any is True

    assert hasattr(log, 'puppet_master_manifest_last')
    assert 'puppet-master' in log.puppet_master_manifest_last['raw_message']
    assert 'manifest' in log.puppet_master_manifest_last['raw_message']
    assert hasattr(log, 'puppet_master_first_last_any')
    assert 'puppet-master' in log.puppet_master_first_last_any['raw_message']


def test_messages_get_after():
    ctx = context_wrap(MESSAGES, path='/var/log/messages')
    log = FakeMessagesClass(ctx)
    assert len(log.lines) == 31

    # Get lines after date
    # Remember, logs with no date are assumed to be in year we give it
    assert len(list(log.get_after(datetime(2017, 3, 27, 3, 39, 46)))) == 3
    # Get subset of lines after date
    pulp = log.get('pulp')  # includes /pulp/ in traceback
    assert len(pulp) == 8
    # Get lines after date, with one keyword
    after = list(log.get_after(datetime(2017, 3, 27, 3, 20, 30), 'pulp'))
    assert len(after) == 5
    # Get lines after date, with one keyword
    after = list(log.get_after(datetime(2017, 3, 27, 3, 40, 30), 'pulp'))
    assert len(after) == 1

    # Get lines after date, with one keywords list
    after = list(log.get_after(datetime(2017, 3, 27, 3, 20, 30), ['pulp', 'ERROR']))
    assert len(after) == 2
    after = list(log.get_after(datetime(2017, 3, 27, 3, 40, 30), 'pulp'))
    assert len(after) == 1
    # No lines are found
    assert list(log.get_after(datetime(2017, 3, 27, 3, 49, 46), 'pulp')) == []

    tmp_time = datetime(2017, 3, 27, 3, 39, 46)
    with pytest.raises(TypeError):
        list(log.get_after(tmp_time, ['type=', False, 'AVC']))
    with pytest.raises(TypeError):
        list(log.get_after(tmp_time, set(['type=', 'AVC'])))


def test_messages_get_after_bad_time_format():
    ctx = context_wrap(MESSAGES, path='/var/log/messages')
    log = FakeMessagesClass(ctx)
    assert len(log.lines) == 31

    # The timestamp format should only contain parts that get_after recognises
    with pytest.raises(ParseException) as exc:
        assert list(log.bad_get_after(datetime(2017, 3, 27, 3, 39, 46))) is None
    assert 'get_after does not understand strptime format ' in str(exc)


MESSAGES_ROLLOVER_YEAR = """
Dec 31 21:43:00 duradm13 [CMA]: Logger failed to open catalog file
Dec 31 22:03:05 duradm13 xinetd[21465]: START: bgssd pid=28021 from=10.20.40.7
Dec 31 22:03:05 duradm13 xinetd[21465]: EXIT: bgssd status=0 pid=28021 duration=0(sec)
Dec 31 23:03:07 duradm13 xinetd[21465]: START: bgssd pid=31307 from=10.20.40.7
Dec 31 23:03:07 duradm13 xinetd[21465]: EXIT: bgssd status=0 pid=31307 duration=0(sec)
Dec 31 23:07:00 duradm13 [CMA]: Logger failed to open catalog file
Jan  1 00:00:00 duradm13 [CMA]: Logger failed to open catalog file
Jan  1 00:03:09 duradm13 xinetd[21465]: START: bgssd pid=2203 from=10.20.40.7
Jan  1 00:03:09 duradm13 xinetd[21465]: EXIT: bgssd status=0 pid=2203 duration=0(sec)
Jan  1 00:11:45 duradm13 xinetd[21465]: START: vnetd pid=2670 from=10.20.40.36
Jan  1 00:11:47 duradm13 xinetd[21465]: START: vnetd pid=2671 from=10.20.40.36
Jan  1 00:11:48 duradm13 xinetd[21465]: EXIT: vnetd status=0 pid=2671 duration=1(sec)
Jan  1 01:00:08 duradm13 xinetd[21465]: START: nrpe pid=6189 from=10.20.40.240
Jan  1 01:00:08 duradm13 xinetd[21465]: EXIT: nrpe status=0 pid=6189 duration=0(sec)
Jan  1 01:00:27 duradm13 xinetd[21465]: START: nrpe pid=6207 from=10.20.40.240
Jan  1 01:00:27 duradm13 xinetd[21465]: EXIT: nrpe status=0 pid=6207 duration=0(sec)
Jan  1 01:00:29 duradm13 xinetd[21465]: START: nrpe pid=6210 from=10.20.40.240
Jan  1 01:00:29 duradm13 xinetd[21465]: EXIT: nrpe status=0 pid=6210 duration=0(sec)
""".strip()


def test_messages_log_time_wrap():
    # Check rollover process for when dates in logs without a year go from
    # December from January.
    ctx = context_wrap(MESSAGES_ROLLOVER_YEAR, path='/var/log/messages')
    log = FakeMessagesClass(ctx)
    assert len(log.lines) == 18

    # If sought date is in January, we should not get logs from December
    found = list(log.get_after(datetime(2017, 1, 1, 1, 0, 0)))
    assert len(found) == 6
    # If sought date is in December, we should get logs from January
    found = list(log.get_after(datetime(2017, 12, 31, 23, 0, 0)))
    assert len(found) == 15


HTTPD_ACCESS_LOG = """
192.168.220.42 - - [14/Feb/2016:03:18:54 -0600] "POST /XMLRPC HTTP/1.1" 200 1381 "-" "rhn.rpclib.py/$Revision$"
192.168.220.42 - - [14/Feb/2016:03:18:54 -0600] "POST /XMLRPC HTTP/1.1" 200 3282 "-" "rhn.rpclib.py/$Revision$"
192.168.220.42 - - [14/Feb/2016:03:18:54 -0600] "POST /XMLRPC HTTP/1.1" 200 163 "-" "rhn.rpclib.py/$Revision$"
192.168.220.42 - - [14/Feb/2016:03:18:54 -0600] "POST /XMLRPC HTTP/1.1" 200 138 "-" "rhn.rpclib.py/$Revision$"
192.168.128.243 - - [14/Feb/2016:03:19:00 -0600] "POST /cobbler_api HTTP/1.1" 200 144 "-" "Java/1.6.0"
192.168.128.243 - - [14/Feb/2016:03:19:00 -0600] "POST /cobbler_api HTTP/1.1" 200 129 "-" "Java/1.6.0"
192.168.128.243 - - [14/Feb/2016:03:20:00 -0600] "POST /cobbler_api HTTP/1.1" 200 144 "-" "Java/1.6.0"
192.168.128.243 - - [14/Feb/2016:03:20:00 -0600] "POST /cobbler_api HTTP/1.1" 200 129 "-" "Java/1.6.0"
"""


class FakeAccessLog(LogFileOutput):
    time_format = '%d/%b/%Y:%H:%M:%S'


def test_logs_with_year():
    ctx = context_wrap(HTTPD_ACCESS_LOG, path='/var/log/httpd/access_log')
    log = FakeAccessLog(ctx)
    assert len(log.lines) == 8

    assert len(list(log.get_after(datetime(2016, 2, 14, 3, 18, 55)))) == 4


DATE_CHANGE_MARIADB_LOG = """
161109  9:25:42 [Warning] SSL error: SSL_CTX_set_default_verify_paths failed
161109  9:25:42 [Note] WSREP: Service disconnected.
161109  9:25:43 [Note] WSREP: Some threads may fail to exit.
161109 14:28:24 InnoDB: Initializing buffer pool, size = 128.0M
161109 14:28:24 InnoDB: Completed initialization of buffer pool
2017-01-03 09:36:17 139651251140544 [Note] MariaDB 10.1.5 started successfully
"""


class FakeMariaDBLog(LogFileOutput):
    time_format = ['%y%m%d %H:%M:%S', '%Y-%m-%d %H:%M:%S']


class DictMariaDBLog(LogFileOutput):
    time_format = {'old': '%y%m%d %H:%M:%S', 'new': '%Y-%m-%d %H:%M:%S'}


class BadClassMariaDBLog(LogFileOutput):
    time_format = FakeAccessLog


def test_logs_with_two_timestamps():
    ctx = context_wrap(DATE_CHANGE_MARIADB_LOG, path='/var/log/mariadb/mariadb.log')
    log = FakeMariaDBLog(ctx)
    assert len(log.lines) == 6

    new_ver = list(log.get_after(datetime(2017, 1, 1, 9, 0, 0)))
    assert len(new_ver) == 1
    assert new_ver[0]['raw_message'] == '2017-01-03 09:36:17 139651251140544 [Note] MariaDB 10.1.5 started successfully'

    dict_log = DictMariaDBLog(ctx)
    new_ver_d = list(dict_log.get_after(datetime(2017, 1, 1, 9, 0, 0)))
    assert len(new_ver_d) == 1
    assert new_ver_d[0]['raw_message'] == '2017-01-03 09:36:17 139651251140544 [Note] MariaDB 10.1.5 started successfully'

    # get_after should only supply a string or list for time_format
    with pytest.raises(ParseException) as exc:
        logerr = BadClassMariaDBLog(ctx)
        assert list(logerr.get_after(datetime(2017, 3, 27, 3, 39, 46))) is None
    assert 'get_after does not recognise time formats of type ' in str(exc)


MILLISECONDS_TOWER_LOG = """
2020-05-28 19:25:36,892 WARNING  django.request Not Found: /api/v1/me/
2020-05-28 19:25:46,944 INFO     awx.api.authentication User admin performed a GET to /api/v2/activity_stream/ through the API
2020-05-28 19:33:03,125 INFO     awx.api.authentication User admin performed a GET to /api/v2/job_templates/7/survey_spec/ through the API
2020-05-28 19:33:03,413 INFO     awx.api.authentication User admin performed a GET to /api/v2/projects/ through the API
""".strip()


class FakeTowerLog(LogFileOutput):
    time_format = '%Y-%m-%d %H:%M:%S,%f'


def test_logs_with_milliseconds():
    ctx = context_wrap(MILLISECONDS_TOWER_LOG, path='/var/log/tower/tower.log')
    log = FakeTowerLog(ctx)
    assert len(log.lines) == 4
    assert len(list(log.get_after(datetime(2020, 5, 28, 19, 25, 46, 944)))) == 3



import doctest

from insights.parsers import rhsm_conf
from insights.tests import context_wrap


CONFIG = """
# Red Hat Subscription Manager Configuration File:

# Unified Entitlement Platform Configuration
[server]
# Server hostname:
hostname = subscription.rhn.redhat.com

# Server prefix:
prefix = /subscription

# Server port:
port = 443

# Set to 1 to disable certificate validation:
insecure = 0

# Set the depth of certs which should be checked
# when validating a certificate
ssl_verify_depth = 3

# an http proxy server to use
proxy_hostname =

# port for http proxy server
proxy_port =

# user name for authenticating to an http proxy, if needed
proxy_user =

# password for basic http proxy auth, if needed
proxy_password =

[rhsm]
# Content base URL:
baseurl= https://cdn.redhat.com

# Server CA certificate location:
ca_cert_dir = /etc/rhsm/ca/

# Default CA cert to use when generating yum repo configs:
repo_ca_cert = %(ca_cert_dir)sredhat-uep.pem

# Where the certificates should be stored
productCertDir = /etc/pki/product
entitlementCertDir = /etc/pki/entitlement
consumerCertDir = /etc/pki/consumer

# Manage generation of yum repositories for subscribed content:
manage_repos = 1

# Refresh repo files with server overrides on every yum command
full_refresh_on_yum = 0

# If set to zero, the client will not report the package profile to
# the subscription management service.
report_package_profile = 1

# The directory to search for subscription manager plugins
pluginDir = /usr/share/rhsm-plugins

# The directory to search for plugin configuration files
pluginConfDir = /etc/rhsm/pluginconf.d

[rhsmcertd]
# Interval to run cert check (in minutes):
certCheckInterval = 240
# Interval to run auto-attach (in minutes):
autoAttachInterval = 1440

""".strip()


def test_doc_examples():
    failed_count, tests = doctest.testmod(
        rhsm_conf,
        globs={'conf': rhsm_conf.RHSMConf(context_wrap(CONFIG))}
    )
    assert failed_count == 0


def test_rhsm_conf():
    result = rhsm_conf.RHSMConf(context_wrap(CONFIG))

    assert result.get('rhsm', 'pluginConfDir') == '/etc/rhsm/pluginconf.d'
    assert result.get('rhsm', 'full_refresh_on_yum') == '0'
    assert result.get('rhsm', 'consumerCertDir') == '/etc/pki/consumer'
    assert result.get('rhsm', 'repo_ca_cert') == '%(ca_cert_dir)sredhat-uep.pem'



import pytest
import sys
import tempfile

from insights.core.hydration import get_all_files
from os import chmod, makedirs
from os.path import join
from shutil import rmtree


def test_get_all_files():
    tmp_dir = tempfile.mkdtemp()

    d = join(tmp_dir, 'sys', 'kernel')
    makedirs(d)
    with open(join(d, 'kexec_crash_size'), "w") as f:
        f.write("ohyeahbaby")

    with open(join(d, 'kexec_loaded'), "w") as f:
        f.write("1")

    assert any(f.endswith("/sys/kernel/kexec_crash_size") for f in get_all_files(tmp_dir))

    rmtree(tmp_dir, ignore_errors=True)


@pytest.mark.skipif(sys.version_info < (3, 6), reason="This issue only occurs on python3+.")
def test_get_all_files_oserror(caplog):
    tmp_dir = tempfile.mkdtemp()

    d = join(tmp_dir, 'sys', 'kernel')
    makedirs(d)
    with open(join(d, 'kexec_crash_size'), "w") as f:
        f.write("ohyeahbaby")

    chmod(d, 0o644)
    chmod(join(tmp_dir, "sys"), 0o677)

    any(f.endswith("/sys/kernel/kexec_crash_size") for f in get_all_files(tmp_dir))
    assert "Errno 13" in caplog.text

    chmod(join(tmp_dir, "sys"), 0o777)
    chmod(d, 0o777)
    rmtree(tmp_dir, ignore_errors=True)



import doctest
import pytest

from insights.core.exceptions import SkipComponent
from insights.parsers import nmcli
from insights.parsers.nmcli import NmcliConnShow, NmcliDevShow, NmcliDevShowSos, NmcliConnShowUuids
from insights.tests import context_wrap

NMCLI_SHOW = """
GENERAL.DEVICE:                         em3
GENERAL.TYPE:                           ethernet
GENERAL.HWADDR:                         B8:2A:72:DE:F8:B9
GENERAL.MTU:                            1500
GENERAL.STATE:                          100 (connected)
GENERAL.CONNECTION:                     em3
GENERAL.CON-PATH:                       /org/freedesktop/NetworkManager/ActiveConnection/1
WIRED-PROPERTIES.CARRIER:               on
IP4.ADDRESS[1]:                         10.16.184.98/22
IP4.GATEWAY:                             10.16.187.254
IP4.DNS[1]:                             10.16.36.29
IP4.DNS[2]:                             10.11.5.19
IP4.DNS[3]:                             10.5.30.160
IP4.DOMAIN[1]:                          khw.lab.eng.bos.example.com
IP6.ADDRESS[1]:                         2620:52:0:10bb:ba2a:72ff:fede:f8b9/64
IP6.ADDRESS[2]:                         fe80::ba2a:72ff:fede:f8b9/64
IP6.GATEWAY:                            fe80:52:0:10bb::fc
IP6.ROUTE[1]:                           dst = 2620:52:0:10bb::/64, nh = ::, mt = 100

GENERAL.DEVICE:                         em1
GENERAL.TYPE:                           ethernet
GENERAL.HWADDR:                         B8:2A:72:DE:F8:BB
GENERAL.MTU:                            1500
GENERAL.STATE:                          100 connected
GENERAL.CONNECTION:                     --
GENERAL.CON-PATH:                       --
WIRED-PROPERTIES.CARRIER:               off

GENERAL.DEVICE:                         em2
GENERAL.TYPE:                           ethernet
GENERAL.HWADDR:                         B8:AA:BB:DE:F8:BC
GENERAL.MTU:                            1500
GENERAL.STATE:                          100 (connected)
GENERAL.CONNECTION:                     --
GENERAL.CON-PATH:                       --
WIRED-PROPERTIES.CARRIER:               off
"""

NMCLI_SHOW_SOS = """
GENERAL.DEVICE:                         em3
GENERAL.TYPE:                           ethernet
GENERAL.HWADDR:                         B8:2A:72:DE:F8:B9
GENERAL.MTU:                            1500
GENERAL.STATE:                          100 (connected)
GENERAL.CONNECTION:                     em3
GENERAL.CON-PATH:                       /org/freedesktop/NetworkManager/ActiveConnection/1
WIRED-PROPERTIES.CARRIER:               on
IP4.ADDRESS[1]:                         10.16.184.98/22
IP4.GATEWAY:                             10.16.187.254
IP4.DNS[1]:                             10.16.36.29
IP4.DNS[2]:                             10.11.5.19
IP4.DNS[3]:                             10.5.30.160
IP4.DOMAIN[1]:                          khw.lab.eng.bos.example.com
IP6.ADDRESS[1]:                         2620:52:0:10bb:ba2a:72ff:fede:f8b9/64
IP6.ADDRESS[2]:                         fe80::ba2a:72ff:fede:f8b9/64
IP6.GATEWAY:                            fe80:52:0:10bb::fc
IP6.ROUTE[1]:                           dst = 2620:52:0:10bb::/64, nh = ::, mt = 100
""".strip()

NMCLI_SHOW_ERROR = """
Error: Option '-l' is unknown, try 'nmcli -help'.
"""

NMCLI_SHOW_ERROR_2 = """
Error: Option '-l' is unknown, try 'nmcli -help'.
Warning: nmcli (1.0.0) and NetworkManager (1.0.6) versions don't match. Use --nocheck to suppress the warning.
"""

STATIC_CONNECTION_SHOW_1 = """
NAME      UUID                                  TYPE      DEVICE
enp0s3    320d4923-c410-4b22-b7e9-afc5f794eecc  ethernet  enp0s3
virbr0    7c7dec66-4a8c-4b49-834a-889194b3b83c  bridge    virbr0
test-net-1  f858b1cc-d149-4de0-93bc-b1826256847a  ethernet  --
""".strip()

STATIC_CONNECTION_SHOW_2 = """
NAME      UUID                                  TYPE      DEVICE
enp0s3    320d4923-c410-4b22-b7e9-afc5f794eecc  ethernet  enp0s3
virbr0    7c7dec66-4a8c-4b49-834a-889194b3b83c  bridge    virbr0
test-net-1  f858b1cc-d149-4de0-93bc-b1826256847a  ethernet  --
test-net-2 f858b1cc-d149-4de0-93bc-b1826256847a  ethernet  --
""".strip()

STATIC_CONNECTION_SHOW_3 = """
Warning: nmcli (1.0.0) and NetworkManager (1.0.6) versions don't match. Use --nocheck to suppress the warning.
NAME           UUID                                  TYPE            DEVICE
enp0s8         00cb8299-feb9-55b6-a378-3fdc720e0bc6  802-3-ethernet  --
enp0s3         bfb4760c-96ce-4a29-9f2e-7427051da943  802-3-ethernet  enp0s3"
""".strip()

STATIC_CONNECTION_SHOW_4 = """
Warning: nmcli (1.0.0) and NetworkManager (1.0.6) versions don't match. Use --nocheck to suppress the warning.
NAME                UUID                                  TYPE      DEVICE
System eth0         5fb06bd0-0bb0-7ffb-45f1-d6edd65f3e03  ethernet  eth0
Wired connection 1  ba12c52b-cd0e-39a0-a95b-643a0859664e  ethernet  eth1
Wired connection 2  fa8d308c-2e00-336c-9dee-2def12e240c7  ethernet  eth2
team0               bf000427-d9f1-432f-819d-257edb86c6fb  team      team0
""".strip()

NMCLI_DEV_SHOW = """
TextFileProvider("'/tmp/insights-fcct09p0/insights-rhel7-box-20191016082653/insights_commands/nmcli_dev_show'")
GENERAL.DEVICE:                         br0
GENERAL.TYPE:                           bridge
GENERAL.HWADDR:                         7A:C3:4C:23:65:8A
GENERAL.MTU:                            1450
GENERAL.STATE:                          100 (connected)
GENERAL.CONNECTION:                     br0
GENERAL.CON-PATH:                       /org/freedesktop/NetworkManager/ActiveConnection/1
IP4.GATEWAY:
IP6.ADDRESS[1]:                         fe80::78c3:4cff:fe23:658a/64
IP6.GATEWAY:

GENERAL.DEVICE:                         enp0s3
GENERAL.TYPE:                           ethernet
GENERAL.HWADDR:                         08:00:27:4A:C5:EF
GENERAL.MTU:                            1500
GENERAL.STATE:                          100 (connected)
GENERAL.CONNECTION:                     enp0s3
GENERAL.CON-PATH:                       /org/freedesktop/NetworkManager/ActiveConnection/0
WIRED-PROPERTIES.CARRIER:               on
IP4.ADDRESS[1]:                         10.0.2.15/24
IP4.GATEWAY:                            10.0.2.2
IP4.DNS[1]:                             10.0.2.3
IP4.DOMAIN[1]:                          redhat.com
IP6.ADDRESS[1]:                         fe80::a00:27ff:fe4a:c5ef/64
IP6.GATEWAY:

GENERAL.DEVICE:                         vxlan10
GENERAL.TYPE:                           vxlan
GENERAL.HWADDR:                         7A:C3:4C:23:65:8A
GENERAL.MTU:                            1450
GENERAL.STATE:                          100 (connected)
GENERAL.CONNECTION:                     vxlan10
GENERAL.CON-PATH:                       /org/freedesktop/NetworkManager/ActiveConnection/2
IP4.GATEWAY:
IP6.GATEWAY:

GENERAL.DEVICE:                         enp0s8
GENERAL.TYPE:                           ethernet
GENERAL.HWADDR:                         08:00:27:45:74:6B
GENERAL.MTU:                            1500
GENERAL.STATE:                          30 (disconnected)
GENERAL.CONNECTION:                     --
GENERAL.CON-PATH:                       --
WIRED-PROPERTIES.CARRIER:               on

GENERAL.DEVICE:                         enp0s9
GENERAL.TYPE:                           ethernet
GENERAL.HWADDR:                         08:00:27:F2:32:1E
GENERAL.MTU:                            1500
GENERAL.STATE:                          30 (disconnected)
GENERAL.CONNECTION:                     --
GENERAL.CON-PATH:                       --
WIRED-PROPERTIES.CARRIER:               on

GENERAL.DEVICE:                         lo
GENERAL.TYPE:                           loopback
GENERAL.HWADDR:                         00:00:00:00:00:00
GENERAL.MTU:                            65536
GENERAL.STATE:                          10 (unmanaged)
GENERAL.CONNECTION:                     --
GENERAL.CON-PATH:                       --
IP4.ADDRESS[1]:                         127.0.0.1/8
IP4.GATEWAY:
IP6.ADDRESS[1]:                         ::1/128
IP6.GATEWAY:
""".strip()

NMCLI_CONN_SHOW_UUID = """
connection.id:                          System eth0
connection.uuid:                        5fb06bd0-0bb0-7ffb-45f1-ffffffffffff
connection.stable-id:                   --
connection.type:                        802-3-ethernet
connection.interface-name:              eth0
connection.autoconnect:                 yes
connection.autoconnect-priority:        0
connection.autoconnect-retries:         -1 (default)
connection.multi-connect:               0 (default)
connection.auth-retries:                -1
connection.timestamp:                   1724660474
connection.read-only:                   no
connection.permissions:                 --
ipv4.dad-timeout:                       -1 (default)
ipv4.dhcp-vendor-class-identifier:      --
ipv4.link-local:                        0 (default)
IP4.DOMAIN[1]:                          gsslab.pek2.redhat.com
DHCP4.OPTION[1]:                        dhcp_client_identifier = 01:52:54:00:fc:b1:cb
DHCP4.OPTION[2]:                        dhcp_lease_time = 86400
""".strip()


def test_nmcli():
    nmcli_obj = NmcliDevShow(context_wrap(NMCLI_SHOW))
    con_dev = nmcli_obj.connected_devices
    assert sorted(con_dev) == sorted(['em1', 'em3', 'em2'])
    assert nmcli_obj['em3']['IP4_GATEWAY'] == "10.16.187.254"
    assert nmcli_obj['em3']['IP4_DNS1'] == "10.16.36.29"
    assert nmcli_obj['em3']['IP6_ROUTE1'] == "dst = 2620:52:0:10bb::/64, nh = ::, mt = 100"
    assert nmcli_obj['em1']['STATE'] == "connected"
    assert nmcli_obj['em1']['CON-PATH'] == "--"
    assert nmcli_obj['em3']['IP6_ADDRESS1'] == "2620:52:0:10bb:ba2a:72ff:fede:f8b9/64"
    assert nmcli_obj['em3']['IP6_ADDRESS2'] == "fe80::ba2a:72ff:fede:f8b9/64"
    assert nmcli_obj['em3']['CON-PATH'] == "/org/freedesktop/NetworkManager/ActiveConnection/1"
    assert len(nmcli_obj['em3']) == 17
    assert len(nmcli_obj['em1']) == 7
    nmcli_obj = NmcliDevShow(context_wrap(NMCLI_DEV_SHOW))
    assert 'IP6_GATEWAY' not in nmcli_obj['lo']
    assert 'IP6_ADDRESS1' in nmcli_obj['lo']
    assert nmcli_obj['lo']['IP6_ADDRESS1'] == '::1/128'
    assert nmcli_obj.data is not None


def test_nmcli_conn_show_uuids():
    nmcli_conn_show_uuid_obj = NmcliConnShowUuids(context_wrap(NMCLI_CONN_SHOW_UUID))
    assert nmcli_conn_show_uuid_obj['connection.uuid'] == "5fb06bd0-0bb0-7ffb-45f1-ffffffffffff"
    assert nmcli_conn_show_uuid_obj['ipv4.dhcp-vendor-class-identifier'] == "--"
    assert nmcli_conn_show_uuid_obj['DHCP4.OPTION[2]'] == "dhcp_lease_time = 86400"

    with pytest.raises(SkipComponent):
        NmcliConnShowUuids(context_wrap(""))


def test_nmcli_sos():
    nmcli_obj = NmcliDevShowSos(context_wrap(NMCLI_SHOW_SOS))
    con_dev = nmcli_obj.connected_devices
    assert sorted(con_dev) == sorted(['em3'])
    assert nmcli_obj['em3']['IP4_GATEWAY'] == "10.16.187.254"
    assert nmcli_obj['em3']['IP4_DNS1'] == "10.16.36.29"
    assert nmcli_obj['em3']['IP6_ROUTE1'] == "dst = 2620:52:0:10bb::/64, nh = ::, mt = 100"
    assert nmcli_obj['em3']['IP6_ADDRESS1'] == "2620:52:0:10bb:ba2a:72ff:fede:f8b9/64"
    assert nmcli_obj['em3']['IP6_ADDRESS2'] == "fe80::ba2a:72ff:fede:f8b9/64"
    assert nmcli_obj['em3']['CON-PATH'] == "/org/freedesktop/NetworkManager/ActiveConnection/1"
    assert len(nmcli_obj['em3']) == 17


def test_static_connection_test_1():
    static_conn = NmcliConnShow(context_wrap(STATIC_CONNECTION_SHOW_1))
    assert static_conn.data[0] == {'NAME': 'enp0s3', 'UUID': '320d4923-c410-4b22-b7e9-afc5f794eecc', 'TYPE': 'ethernet', 'DEVICE': 'enp0s3'}
    assert static_conn.disconnected_connection == ["test-net-1"]


def test_static_connection_test_2():
    static_conn = NmcliConnShow(context_wrap(STATIC_CONNECTION_SHOW_2))
    assert static_conn.disconnected_connection == ["test-net-1", "test-net-2"]


def test_static_connection_test_3():
    static_conn = NmcliConnShow(context_wrap(STATIC_CONNECTION_SHOW_3))
    assert static_conn.disconnected_connection == ["enp0s8"]


def test_static_connection_test_4():
    static_conn = NmcliConnShow(context_wrap(STATIC_CONNECTION_SHOW_4))
    assert static_conn.data[1] == {'NAME': 'Wired connection 1', 'UUID': 'ba12c52b-cd0e-39a0-a95b-643a0859664e', 'TYPE': 'ethernet', 'DEVICE': 'eth1'}
    assert static_conn.data[3] == {'NAME': 'team0', 'UUID': 'bf000427-d9f1-432f-819d-257edb86c6fb', 'TYPE': 'team', 'DEVICE': 'team0'}


def test_nmcli_dev_show_ab():
    with pytest.raises(SkipComponent):
        NmcliDevShow(context_wrap(''))

    with pytest.raises(SkipComponent):
        NmcliDevShow(context_wrap('GENERAL.TYPE: ethernet'))

    with pytest.raises(SkipComponent):
        NmcliDevShow(context_wrap('Error'))


def test_nmcli_doc_examples():
    env = {
        'nmcli_obj': NmcliDevShow(context_wrap(NMCLI_SHOW)),
        'nmcli_obj_sos': NmcliDevShowSos(context_wrap(NMCLI_SHOW_SOS)),
        'static_conn': NmcliConnShow(context_wrap(STATIC_CONNECTION_SHOW_1)),
        'conn_show_uuids': NmcliConnShowUuids(context_wrap(NMCLI_CONN_SHOW_UUID))
    }
    failed, total = doctest.testmod(nmcli, globs=env)
    assert failed == 0


def test_nmcli_exceptions():
    with pytest.raises(SkipComponent) as exc:
        nmcli_obj = NmcliConnShow(context_wrap(NMCLI_SHOW_ERROR))
        nmcli_obj = NmcliConnShow(context_wrap(NMCLI_SHOW_ERROR_2))
        assert nmcli_obj is None
    assert 'Invalid Contents!' in str(exc)



"""
JournalCtl - command ``journalctl xxx``
=======================================

This module contains the following parsers:

JournalAll - command ``journalctl --no-pager``
----------------------------------------------
JournalHeader - command ``journalctl --no-pager --header``
----------------------------------------------------------
JournalSinceBoot - command ``journalctl --no-pager --boot``
-----------------------------------------------------------

"""


from insights.core.plugins import parser
from insights.specs import Specs
from insights.core import Syslog


@parser(Specs.journal_all)
class JournalAll(Syslog):
    """
    Handle the output of ``journalctl --no-pager`` command.  Uses the ``Syslog`` class
    parser functionality - see the base class for more details.

    Sample log lines::

        -- Logs begin at Wed 2017-02-08 15:18:00 CET, end at Tue 2017-09-19 09:25:27 CEST. --
        May 18 15:13:34 lxc-rhel68-sat56 jabberd/sm[11057]: session started: jid=rhn-dispatcher-sat@lxc-rhel6-sat56.redhat.com/superclient
        May 18 15:13:36 lxc-rhel68-sat56 wrapper[11375]: --> Wrapper Started as Daemon
        May 18 15:13:36 lxc-rhel68-sat56 wrapper[11375]: Launching a JVM...
        May 18 15:24:28 lxc-rhel68-sat56 yum[11597]: Installed: lynx-2.8.6-27.el6.x86_64
        May 18 15:36:19 lxc-rhel68-sat56 yum[11954]: Updated: sos-3.2-40.el6.noarch

    .. note::
        Because journal timestamps by default have no year,
        the year of the logs will be inferred from the year in your timestamp.
        This will also work around December/January crossovers.

    Examples:
        >>> type(JournalAll)
        <class 'insights.parsers.journalctl.JournalAll'>
        >>> len(JournalAll.lines)
        10
        >>> bona_list = JournalAll.get('(root) LIST (root)')
        >>> bona_list[0].get('message')
        '(root) LIST (root)'
    """
    pass


@parser(Specs.journal_header)
class JournalHeader(Syslog):
    """
    Parses the output of command "journalctl --no-pager --header", this command shows
    internal header information of the journal fields accessed. Although its output is
    not kind of Logs, we used the `Syslog` as the base class to parser it.

    Sample Output::

        File Path: /run/log/journal/6bdaf92aa0754b53acbb1dbff7127e2b/system.journal
        File ID: b1390ea69aa747e9ac5c597835c3c562
        Machine ID: 6bdaf92aa0754b53acbb1dbff7127e2b
        Boot ID: 082ada53f8184c4896c73101ad793eb5
        Sequential Number ID: 435b0e30f47a46d8a2a2f9a42eae0aaf
        State: ONLINE
        Compatible Flags:
        Incompatible Flags: COMPRESSED-LZ4
        Header size: 240
        Arena size: 8388368
        Data Hash Table Size: 19904
        Field Hash Table Size: 333
        Rotate Suggested: no
        Head Sequential Number: 74388 (12294)
        Tail Sequential Number: 81647 (13eef)
        Head Realtime Timestamp: Mon 2022-08-15 12:01:10 CST (5e63fae9e6e58)
        Tail Realtime Timestamp: Wed 2022-08-17 18:38:48 CST (5e66d7852bb3e)
        Tail Monotonic Timestamp: 1month 2w 3d 14h 19min 3.733s (3c647d7ce0f)
        Objects: 19073
        Entry Objects: 7260
        Data Objects: 9297
        Data Hash Table Fill: 46.7%
        Field Objects: 52
        Field Hash Table Fill: 15.6%
        Tag Objects: 0
        Entry Array Objects: 2462
        Disk usage: 8.0M

    Examples:
        >>> type(journal_header)
        <class 'insights.parsers.journalctl.JournalHeader'>
        >>> result_list = journal_header.get('File Path')
        >>> len(result_list)
        1
        >>> result_list[0].get('raw_message')
        'File Path: /run/log/journal/6bdaf92aa0754b53acbb1dbff7127e2b/system.journal'
    """

    pass


@parser(Specs.journal_since_boot)
class JournalSinceBoot(Syslog):
    """
    Handle the output of ``journalctl --no-pager --boot`` command.  Uses
    the ``Syslog`` class parser functionality - see the base class for more
    details.

    Sample log lines::

        -- Logs begin at Wed 2017-02-08 15:18:00 CET, end at Tue 2017-09-19 09:25:27 CEST. --
        May 18 15:13:34 lxc-rhel68-sat56 jabberd/sm[11057]: session started: jid=rhn-dispatcher-sat@lxc-rhel6-sat56.redhat.com/superclient
        May 18 15:13:36 lxc-rhel68-sat56 wrapper[11375]: --> Wrapper Started as Daemon
        May 18 15:13:36 lxc-rhel68-sat56 wrapper[11375]: Launching a JVM...
        May 18 15:24:28 lxc-rhel68-sat56 yum[11597]: Installed: lynx-2.8.6-27.el6.x86_64
        May 18 15:36:19 lxc-rhel68-sat56 yum[11954]: Updated: sos-3.2-40.el6.noarch

    Note:
        Because journal timestamps by default have no year,
        the year of the logs will be inferred from the year in your timestamp.
        This will also work around December/January crossovers.

    Examples:
        >>> type(JournalSinceBoot)
        <class 'insights.parsers.journalctl.JournalSinceBoot'>
        >>> len(JournalSinceBoot.lines)
        10
        >>> bona_list = JournalSinceBoot.get('(root) LIST (root)')
        >>> bona_list[0].get('message')
        '(root) LIST (root)'
    """
    pass



import pytest

from collections import defaultdict

from insights.core import filters
from insights.core.exceptions import SkipComponent
from insights.parsers.fstab import FSTab
from insights.specs import Specs
from insights.specs.datasources.ls import (
    list_with_la,
    list_with_la_filtered,
    list_with_lan,
    list_with_lan_filtered,
    list_with_lanL,
    list_with_lanR,
    list_with_lanRL,
    list_with_laRZ,
    list_with_laZ,
)
from insights.tests import context_wrap


FSTAB_CONTEXT = """
/dev/mapper/rhel_rhel8-root /                       xfs     defaults        0 0
/dev/vda1 /boot                   xfs     defaults        0 0
/dev/mapper/rhel_rhel8-swap none                    swap    defaults        0 0
/dev/mapper/rhel_rhel7-hana1 /hana/data/rhel7-hana1                       xfs     defaults        0 0
/dev/mapper/rhel_rhel7-hana2 /hana/data/rhel7-hana2                       xfs     defaults        0 0
/dev/mapper/rhel_rhel7-hana3 /hana/data/rhel7-hana3                       xfs     defaults        0 0
""".strip()


def setup_function(func):
    if func is test_la_empty:
        pass
    if func is test_la:
        filters.add_filter(Specs.ls_la_dirs, ["/"])
    if func is test_la_filterd:
        filters.add_filter(Specs.ls_la_filtered_dirs, ["/", '/boot'])
    if func is test_lan:
        filters.add_filter(Specs.ls_lan_dirs, ["/", '/boot'])
    if func is test_lan_filterd:
        filters.add_filter(Specs.ls_lan_filtered_dirs, ["/"])
    if func is test_lanL:
        filters.add_filter(Specs.ls_lanL_dirs, ["/", '/boot'])
    if func is test_lanR:
        filters.add_filter(Specs.ls_lanR_dirs, ["/"])
    if func is test_lanRL:
        filters.add_filter(Specs.ls_lanRL_dirs, ["/", '/boot'])
    if func is test_lanRZ:
        filters.add_filter(Specs.ls_laRZ_dirs, ['/boot'])
    if func is test_lanZ:
        filters.add_filter(Specs.ls_laZ_dirs, ["/", '/mnt'])
    if func is test_lan_with_fstab_mounted_filter:
        filters.add_filter(Specs.ls_lan_dirs, ["/", '/boot', 'fstab_mounted.dirs'])


def teardown_function(func):
    filters._CACHE = {}
    filters.FILTERS = defaultdict(set)


def test_la_empty():
    with pytest.raises(SkipComponent):
        list_with_la({})


def test_la():
    ret = list_with_la({})
    assert ret == '/ _non_existing_'


def test_la_filterd():
    ret = list_with_la_filtered({})
    assert ret == '/ /boot'


def test_lan():
    ret = list_with_lan({})
    assert ret == '/ /boot'


def test_lan_filterd():
    ret = list_with_lan_filtered({})
    assert ret == '/ _non_existing_'


def test_lanL():
    ret = list_with_lanL({})
    assert ret == '/ /boot'


def test_lanR():
    ret = list_with_lanR({})
    assert '/' == ret


def test_lanRL():
    ret = list_with_lanRL({})
    assert ret == '/ /boot'


def test_lanRZ():
    ret = list_with_laRZ({})
    assert ret == '/boot'


def test_lanZ():
    ret = list_with_laZ({})
    assert ret == '/ /mnt'


def test_lan_with_fstab_mounted_filter():
    fstab = FSTab(context_wrap(FSTAB_CONTEXT))
    broker = {FSTab: fstab}
    ret = list_with_lan(broker)
    assert ret == '/ /boot /hana/data'



from insights.parsers.route import Route
from insights.tests import context_wrap


ROUTE = '''
Kernel IP routing table
Destination     Gateway         Genmask         Flags Metric Ref    Use Iface
10.66.208.0     0.0.0.0         255.255.255.0   U     0      0        0 eth0
169.254.0.0     0.0.0.0         255.255.0.0     U     1002   0        0 eth0
0.0.0.0         10.66.208.254   0.0.0.0         UG    0      0        0 eth0
'''


def test_route():
    route_info = Route(context_wrap(ROUTE))

    for route in route_info:
        assert route == {'Destination': '10.66.208.0',
                             'Gateway': '0.0.0.0',
                             'Genmask': '255.255.255.0',
                             'Flags': 'U',
                             'Metric': '0',
                             'Ref': '0',
                             'Use': '0',
                             'Iface': 'eth0'}
        break
    assert '169.254.0.0' in route_info



from insights.combiners.ps import Ps
from insights.core import dr
from insights.core.plugins import datasource, condition, parser
from insights.core.spec_factory import DatasourceProvider, RegistryPoint, SpecSet
from insights.parsers.ps import PsAux
from insights.specs import Specs


class RegistrySpecs(SpecSet):
    simple_spec = RegistryPoint()
    first_spec_with_dep = RegistryPoint()
    second_spec_with_dep = RegistryPoint()


@datasource()
def simple_spec_imp(broker):
    return DatasourceProvider("some data", "/path_1")


@datasource()
def dependency_ds(broker):
    return DatasourceProvider("dependency data", "/path_2")


@datasource(dependency_ds)
def first_spec_with_dep_imp(broker):
    return DatasourceProvider("some data", "/path_3")


@datasource(dependency_ds)
def second_spec_with_dep_imp(broker):
    return DatasourceProvider("some data", "/path_4")


class LocalDefaultSpecs(RegistrySpecs):
    simple_spec = simple_spec_imp
    first_spec_with_dep = first_spec_with_dep_imp
    second_spec_with_dep = second_spec_with_dep_imp


@parser(RegistrySpecs.simple_spec)
def simple_spec_condition(spec):
    return True


@condition(RegistrySpecs.first_spec_with_dep, RegistrySpecs.second_spec_with_dep)
def multiple_spec_condition(spec1, spec2):
    return True


def test_is_registry_point():
    assert dr.is_registry_point(RegistrySpecs.simple_spec)
    assert not dr.is_registry_point(LocalDefaultSpecs.simple_spec)


def test_get_registry_points_simple():
    specs = dr.get_registry_points(LocalDefaultSpecs.simple_spec)
    assert len(specs) == 1
    assert list(specs)[0] == RegistrySpecs.simple_spec

    specs = dr.get_registry_points(LocalDefaultSpecs.first_spec_with_dep)
    assert len(specs) == 1
    assert list(specs)[0] == RegistrySpecs.first_spec_with_dep

    specs = dr.get_registry_points(simple_spec_condition)
    assert len(specs) == 1
    assert list(specs)[0] == RegistrySpecs.simple_spec

    specs = dr.get_registry_points(Specs.ps_aux)
    assert len(specs) == 1
    assert list(specs)[0] == Specs.ps_aux

    specs = dr.get_registry_points(PsAux)
    assert len(specs) == 1
    assert list(specs)[0] == Specs.ps_aux


def test_get_registry_points_multiple_specs():
    specs = dr.get_registry_points(multiple_spec_condition)
    assert len(specs) == 2
    for spec in [RegistrySpecs.first_spec_with_dep, RegistrySpecs.second_spec_with_dep]:
        assert spec in specs

    specs = dr.get_registry_points(Ps)
    assert len(specs) == 6
    for spec in [Specs.ps_aux, Specs.ps_auxww, Specs.ps_auxcww, Specs.ps_ef,
                 Specs.ps_auxcww, Specs.ps_eo_cmd]:
        assert spec in specs



from insights.core.exceptions import ContentException, ParseException
from insights.parsers.numeric_user_group_name import NumericUserGroupName
from insights.tests import context_wrap

ZERO = """
/etc/passwd:0
/etc/group:0
""".strip()

NONZERO_1 = """
/etc/passwd:3
/etc/group:0
""".strip()

NONZERO_2 = """
/etc/passwd:0
/etc/group:4
""".strip()

NONZERO_3 = """
/etc/passwd:1
/etc/group:1
""".strip()

UNSTRIPPED_ZERO = """

/etc/passwd:0
/etc/group:0

"""
UNSTRIPPED_NONZERO_1 = """

/etc/passwd:3
/etc/group:0

"""
UNSTRIPPED_NONZERO_2 = """

/etc/passwd:0
/etc/group:4

"""
UNSTRIPPED_NONZERO_3 = """

/etc/passwd:1
/etc/group:1

"""

# should produce no result
ERRORS_1 = """

/etc/passwd:1
/bin/grep: /etc/group: No such file or directory

"""

ERRORS_2 = """

/bin/grep: /etc/passwd: No such file or directory
/etc/group:1

"""

ERRORS_3 = """

/bin/grep: /etc/passwd: No such file or directory
/etc/group:0

"""

# should produce no result
ERRORS_4 = """

/bin/grep: /etc/passwd: No such file or directory
/bin/grep: /etc/group: No such file or directory

"""

# should produce no result
ERRORS_5 = """

/bin/grep: No such file or directory

"""

# Should produce a valid result because it contains valid lines.
# The last valid lines are used.
# /etc/passwd:1
# /etc/group:1
ERRORS_6 = """

/etc/passwd: d:
/etc/group: No such file or directory
/bin/group:7
/bin/etc/group:7
/bin/grep: No such file or directory
# should be ignored:
/etc/passwd:0
# valid output for /etc/passwd on the following line:
/etc/passwd:1
/bin/group:4
/bin/gry
# should be ignored
/etc/group:0
# valid output for /etc/group on the following line:
/etc/group:1
/bin/grep: /etc/passwd:3
"""


def test_numeric_user_group_name_1():
    for data in [ZERO, UNSTRIPPED_ZERO]:
        numeric_user_group_name = NumericUserGroupName(context_wrap(data))
        assert not numeric_user_group_name.has_numeric_user_or_group


def test_numeric_user_group_name_2():
    for data in [NONZERO_1, NONZERO_2, NONZERO_3, UNSTRIPPED_NONZERO_1, UNSTRIPPED_NONZERO_2,
                 UNSTRIPPED_NONZERO_3, ERRORS_6]:
        numeric_user_group_name = NumericUserGroupName(context_wrap(data))
        assert numeric_user_group_name.has_numeric_user_or_group


def test_numeric_user_group_name_6():
    for data in [NONZERO_1, UNSTRIPPED_NONZERO_1]:
        numeric_user_group_name = NumericUserGroupName(context_wrap(data))
        assert numeric_user_group_name.nr_numeric_user == 3
        assert numeric_user_group_name.nr_numeric_group == 0


def test_numeric_user_group_name_7():
    for data in [NONZERO_2, UNSTRIPPED_NONZERO_2]:
        numeric_user_group_name = NumericUserGroupName(context_wrap(data))
        assert numeric_user_group_name.nr_numeric_user == 0
        assert numeric_user_group_name.nr_numeric_group == 4


def test_numeric_user_group_name_8():
    for data in [NONZERO_3, UNSTRIPPED_NONZERO_3, ERRORS_6]:
        numeric_user_group_name = NumericUserGroupName(context_wrap(data))
        assert numeric_user_group_name.nr_numeric_user == 1
        assert numeric_user_group_name.nr_numeric_group == 1


def test_numeric_user_group_name_9():
    for data in [ERRORS_1, ERRORS_2, ERRORS_3, ERRORS_4]:
        try:
            NumericUserGroupName(context_wrap(data))
            assert False
        except ParseException:
            assert True


def test_numeric_user_group_name_10():
    try:
        NumericUserGroupName(context_wrap(ERRORS_5))
        assert False
    except ContentException:
        assert True



r"""
Installed product IDs
=====================

InstalledProductIDs - command ``find /etc/pki/product-default/ /etc/pki/product/ -name '*pem' -exec rct cat-cert --no-content '{}' \;``
---------------------------------------------------------------------------------------------------------------------------------------

This module provides a parser for information about certificates for
Red Hat product subscriptions.

"""
from insights.core.filters import add_filter
from insights.specs import Specs
from .. import parser, CommandParser

add_filter(Specs.subscription_manager_installed_product_ids, ['ID:', 'Product Certificate', 'Path:'])


@parser(Specs.subscription_manager_installed_product_ids)
class InstalledProductIDs(CommandParser):
    r"""
    Parses the output of the comand::

        find /etc/pki/product-default/ /etc/pki/product/ -name '*pem' -exec rct cat-cert --no-content '{}' \;

    Sample output from the unfiltered command looks like::

        +-------------------------------------------+
        Product Certificate
        +-------------------------------------------+

        Certificate:
            Path: /etc/pki/product-default/69.pem
            Version: 1.0
            Serial: 12750047592154749739
            Start Date: 2017-06-28 18:05:10+00:00
            End Date: 2037-06-23 18:05:10+00:00

        Subject:
            CN: Red Hat Product ID [4f9995e0-8dc4-4b4f-acfe-4ef1264b94f3]

        Issuer:
            C: US
            CN: Red Hat Entitlement Product Authority
            O: Red Hat, Inc.
            OU: Red Hat Network
            ST: North Carolina
            emailAddress: ca-support@redhat.com

        Product:
            ID: 69
            Name: Red Hat Enterprise Linux Server
            Version: 7.4
            Arch: x86_64
            Tags: rhel-7,rhel-7-server
            Brand Type:
            Brand Name:


        +-------------------------------------------+
            Product Certificate
        +-------------------------------------------+

        Certificate:
            Path: /etc/pki/product/69.pem
            Version: 1.0
            Serial: 12750047592154751271
            Start Date: 2018-04-13 11:23:50+00:00
            End Date: 2038-04-08 11:23:50+00:00

        Subject:
            CN: Red Hat Product ID [f3c92a95-26be-4bdf-800f-02c044503896]

        Issuer:
            C: US
            CN: Red Hat Entitlement Product Authority
            O: Red Hat, Inc.
            OU: Red Hat Network
            ST: North Carolina
            emailAddress: ca-support@redhat.com

        Product:
            ID: 69
            Name: Red Hat Enterprise Linux Server
            Version: 7.6
            Arch: x86_64
            Tags: rhel-7,rhel-7-server
            Brand Type:
            Brand Name:

    Filters have been added to the parser so that only the filtered lines will be collected.

    Attributes:
        ids (set): set of strings of the unique product IDs
        product_certs(list): list of dicts of the product certificates key-value pairs split by colon,
            the key is transferred to lowercase format concatenated by an underscore if it contains whitespace

    Examples:
        >>> type(products)
        <class 'insights.parsers.installed_product_ids.InstalledProductIDs'>
        >>> list(products.ids)
        ['69']
        >>> products.product_certs[0]
        {'path': '/etc/pki/product-default/69.pem', 'id': '69'}

    """
    def parse_content(self, content):
        """ Parse command output """
        self.ids = set()
        self.product_certs = []
        one_file_data = None
        for line in content:
            line = line.strip()
            # different file delimiter
            if line == 'Product Certificate':
                if one_file_data:
                    self.product_certs.append(one_file_data)
                one_file_data = {}
            elif one_file_data is not None and ':' in line:
                name, value = line.split(':', 1)
                lower_name_with_underscore = '_'.join([item.strip().lower() for item in name.split()])
                one_file_data[lower_name_with_underscore] = value.strip()
        if one_file_data:
            # add the last file data
            self.product_certs.append(one_file_data)
        self.ids = set([item['id'] for item in self.product_certs])
